@@ -1,6 +1,6 @@
-/* Copyright (c) 2015-2023 The Khronos Group Inc.
- * Copyright (c) 2015-2023 Valve Corporation
- * Copyright (c) 2015-2023 LunarG, Inc.
+/* Copyright (c) 2015-2024 The Khronos Group Inc.
+ * Copyright (c) 2015-2024 Valve Corporation
+ * Copyright (c) 2015-2024 LunarG, Inc.
  * Modifications Copyright (C) 2020 Advanced Micro Devices, Inc. All rights reserved.
  * Modifications Copyright (C) 2022 RasterGrid Kft.
  *
@@ -18,7 +18,8 @@
  */
 
 #include "best_practices/best_practices_validation.h"
-#include "best_practices/best_practices_error_enums.h"
+#include "best_practices/bp_state.h"
+#include "state_tracker/buffer_state.h"
 
 void BestPractices::PreCallRecordAllocateMemory(VkDevice device, const VkMemoryAllocateInfo* pAllocateInfo,
                                                 const VkAllocationCallbacks* pAllocator, VkDeviceMemory* pMemory,
@@ -41,13 +42,14 @@ bool BestPractices::PreCallValidateAllocateMemory(VkDevice device, const VkMemor
     bool skip = false;
 
     if ((Count<vvl::DeviceMemory>() + 1) > kMemoryObjectWarningLimit) {
-        skip |= LogPerformanceWarning(kVUID_BestPractices_AllocateMemory_TooManyObjects, device, error_obj.location,
+        skip |= LogPerformanceWarning("BestPractices-vkAllocateMemory-too-many-objects", device, error_obj.location,
                                       "This app has over %" PRIu32 " memory objects.", kMemoryObjectWarningLimit);
     }
 
     if (pAllocateInfo->allocationSize < kMinDeviceAllocationSize) {
-        skip |= LogPerformanceWarning(kVUID_BestPractices_AllocateMemory_SmallAllocation, device, error_obj.location,
-                                      "Allocating a VkDeviceMemory of size %" PRIu64
+        skip |= LogPerformanceWarning("BestPractices-vkAllocateMemory-small-allocation", device,
+                                      error_obj.location.dot(Field::pAllocateInfo).dot(Field::allocationSize),
+                                      "is %" PRIu64
                                       ". This is a very small allocation (current "
                                       "threshold is %" PRIu64
                                       " bytes). "
@@ -59,7 +61,7 @@ bool BestPractices::PreCallValidateAllocateMemory(VkDevice device, const VkMemor
         if (!IsExtEnabled(device_extensions.vk_ext_pageable_device_local_memory) &&
             !vku::FindStructInPNextChain<VkMemoryPriorityAllocateInfoEXT>(pAllocateInfo->pNext)) {
             skip |= LogPerformanceWarning(
-                kVUID_BestPractices_AllocateMemory_SetPriority, device, error_obj.location,
+                "BestPractices-NVIDIA-AllocateMemory-SetPriority", device, error_obj.location,
                 "%s Use VkMemoryPriorityAllocateInfoEXT to provide the operating system information on the allocations that "
                 "should stay in video memory and which should be demoted first when video memory is limited. "
                 "The highest priority should be given to GPU-written resources like color attachments, depth attachments, "
@@ -87,7 +89,7 @@ bool BestPractices::PreCallValidateAllocateMemory(VkDevice device, const VkMemor
                 const auto time_delta = std::chrono::duration_cast<std::chrono::milliseconds>(now - latest_event->time);
                 if (time_delta < std::chrono::milliseconds{5}) {
                     skip |= LogPerformanceWarning(
-                        kVUID_BestPractices_AllocateMemory_ReuseAllocations, device, error_obj.location,
+                        "BestPractices-NVIDIA-AllocateMemory-ReuseAllocations", device, error_obj.location,
                         "%s Reuse memory allocations instead of releasing and reallocating. A memory allocation "
                         "has just been released, and it could have been reused in place of this allocation.",
                         VendorSpecificTag(kBPVendorNVIDIA));
@@ -96,7 +98,7 @@ bool BestPractices::PreCallValidateAllocateMemory(VkDevice device, const VkMemor
                     const uint32_t milliseconds = static_cast<uint32_t>(time_delta.count() % 1000);
 
                     skip |= LogPerformanceWarning(
-                        kVUID_BestPractices_AllocateMemory_ReuseAllocations, device, error_obj.location,
+                        "BestPractices-NVIDIA-AllocateMemory-ReuseAllocations", device, error_obj.location,
                         "%s Reuse memory allocations instead of releasing and reallocating. A memory allocation has been released "
                         "%" PRIu32 ".%03" PRIu32 " seconds ago, and it could have been reused in place of this allocation.",
                         VendorSpecificTag(kBPVendorNVIDIA), seconds, milliseconds);
@@ -114,13 +116,14 @@ void BestPractices::PreCallRecordFreeMemory(VkDevice device, VkDeviceMemory memo
                                             const RecordObject& record_obj) {
     if (memory != VK_NULL_HANDLE && VendorCheckEnabled(kBPVendorNVIDIA)) {
         auto mem_info = Get<vvl::DeviceMemory>(memory);
+        ASSERT_AND_RETURN(mem_info);
 
         // Exclude memory free events on dedicated allocations, or imported/exported allocations.
         if (!mem_info->IsDedicatedBuffer() && !mem_info->IsDedicatedImage() && !mem_info->IsExport() && !mem_info->IsImport()) {
             MemoryFreeEvent event;
             event.time = std::chrono::high_resolution_clock::now();
-            event.memory_type_index = mem_info->alloc_info.memoryTypeIndex;
-            event.allocation_size = mem_info->alloc_info.allocationSize;
+            event.memory_type_index = mem_info->allocate_info.memoryTypeIndex;
+            event.allocation_size = mem_info->allocate_info.allocationSize;
 
             WriteLockGuard guard{memory_free_events_lock_};
             memory_free_events_.push_back(event);
@@ -132,16 +135,17 @@ void BestPractices::PreCallRecordFreeMemory(VkDevice device, VkDeviceMemory memo
 
 bool BestPractices::PreCallValidateFreeMemory(VkDevice device, VkDeviceMemory memory, const VkAllocationCallbacks* pAllocator,
                                               const ErrorObject& error_obj) const {
-    if (memory == VK_NULL_HANDLE) return false;
     bool skip = false;
+    if (memory == VK_NULL_HANDLE) return skip;
 
     auto mem_info = Get<vvl::DeviceMemory>(memory);
+    ASSERT_AND_RETURN_SKIP(mem_info);
 
     for (const auto& item : mem_info->ObjectBindings()) {
         const auto& obj = item.first;
-        const LogObjectList objlist(device, obj, mem_info->deviceMemory());
+        const LogObjectList objlist(device, obj, mem_info->Handle());
         skip |= LogWarning(layer_name.c_str(), objlist, error_obj.location, "VK Object %s still has a reference to mem obj %s.",
-                           FormatHandle(obj).c_str(), FormatHandle(mem_info->deviceMemory()).c_str());
+                           FormatHandle(obj).c_str(), FormatHandle(mem_info->Handle()).c_str());
     }
 
     return skip;
@@ -151,15 +155,16 @@ bool BestPractices::ValidateBindBufferMemory(VkBuffer buffer, VkDeviceMemory mem
     bool skip = false;
     auto buffer_state = Get<vvl::Buffer>(buffer);
     auto mem_state = Get<vvl::DeviceMemory>(memory);
+    ASSERT_AND_RETURN_SKIP(mem_state && buffer_state);
 
-    if (mem_state && mem_state->alloc_info.allocationSize == buffer_state->createInfo.size &&
-        mem_state->alloc_info.allocationSize < kMinDedicatedAllocationSize) {
-        skip |= LogPerformanceWarning(kVUID_BestPractices_SmallDedicatedAllocation, device, loc,
-                                      "%s: Trying to bind %s to a memory block which is fully consumed by the buffer. "
+    if (mem_state->allocate_info.allocationSize == buffer_state->create_info.size &&
+        mem_state->allocate_info.allocationSize < kMinDedicatedAllocationSize) {
+        skip |= LogPerformanceWarning("BestPractices-vkBindBufferMemory-small-dedicated-allocation", device, loc,
+                                      "Trying to bind %s to a memory block which is fully consumed by the buffer. "
                                       "The required size of the allocation is %" PRIu64
                                       ", but smaller buffers like this should be sub-allocated from "
                                       "larger memory blocks. (Current threshold is %" PRIu64 " bytes.)",
-                                      loc.Message().c_str(), FormatHandle(buffer).c_str(), mem_state->alloc_info.allocationSize,
+                                      FormatHandle(buffer).c_str(), mem_state->allocate_info.allocationSize,
                                       kMinDedicatedAllocationSize);
     }
 
@@ -198,15 +203,16 @@ bool BestPractices::ValidateBindImageMemory(VkImage image, VkDeviceMemory memory
     bool skip = false;
     auto image_state = Get<vvl::Image>(image);
     auto mem_state = Get<vvl::DeviceMemory>(memory);
+    ASSERT_AND_RETURN_SKIP(mem_state && image_state);
 
-    if (mem_state->alloc_info.allocationSize == image_state->requirements[0].size &&
-        mem_state->alloc_info.allocationSize < kMinDedicatedAllocationSize) {
-        skip |= LogPerformanceWarning(kVUID_BestPractices_SmallDedicatedAllocation, device, loc,
-                                      "%s: Trying to bind %s to a memory block which is fully consumed by the image. "
+    if (mem_state->allocate_info.allocationSize == image_state->requirements[0].size &&
+        mem_state->allocate_info.allocationSize < kMinDedicatedAllocationSize) {
+        skip |= LogPerformanceWarning("BestPractices-vkBindImageMemory-small-dedicated-allocation", device, loc,
+                                      "Trying to bind %s to a memory block which is fully consumed by the image. "
                                       "The required size of the allocation is %" PRIu64
                                       ", but smaller images like this should be sub-allocated from "
                                       "larger memory blocks. (Current threshold is %" PRIu64 " bytes.)",
-                                      loc.Message().c_str(), FormatHandle(image).c_str(), mem_state->alloc_info.allocationSize,
+                                      FormatHandle(image).c_str(), mem_state->allocate_info.allocationSize,
                                       kMinDedicatedAllocationSize);
     }
 
@@ -214,7 +220,7 @@ bool BestPractices::ValidateBindImageMemory(VkImage image, VkDeviceMemory memory
     // make sure this type is actually used.
     // This warning will only trigger if this layer is run on a platform that supports LAZILY_ALLOCATED_BIT
     // (i.e.most tile - based renderers)
-    if (image_state->createInfo.usage & VK_IMAGE_USAGE_TRANSIENT_ATTACHMENT_BIT) {
+    if (image_state->create_info.usage & VK_IMAGE_USAGE_TRANSIENT_ATTACHMENT_BIT) {
         bool supports_lazy = false;
         uint32_t suggested_type = 0;
 
@@ -228,15 +234,15 @@ bool BestPractices::ValidateBindImageMemory(VkImage image, VkDeviceMemory memory
             }
         }
 
-        uint32_t allocated_properties = phys_dev_mem_props.memoryTypes[mem_state->alloc_info.memoryTypeIndex].propertyFlags;
+        uint32_t allocated_properties = phys_dev_mem_props.memoryTypes[mem_state->allocate_info.memoryTypeIndex].propertyFlags;
 
         if (supports_lazy && (allocated_properties & VK_MEMORY_PROPERTY_LAZILY_ALLOCATED_BIT) == 0) {
             skip |= LogPerformanceWarning(
-                kVUID_BestPractices_NonLazyTransientImage, device, loc,
-                "%s: Attempting to bind memory type %u to VkImage which was created with TRANSIENT_ATTACHMENT_BIT,"
+                "BestPractices-vkBindImageMemory-non-lazy-transient-image", device, loc,
+                "ttempting to bind memory type %u to VkImage which was created with TRANSIENT_ATTACHMENT_BIT,"
                 "but this memory type is not LAZILY_ALLOCATED_BIT. You should use memory type %u here instead to save "
                 "%" PRIu64 " bytes of physical memory.",
-                loc.Message().c_str(), mem_state->alloc_info.memoryTypeIndex, suggested_type, image_state->requirements[0].size);
+                mem_state->allocate_info.memoryTypeIndex, suggested_type, image_state->requirements[0].size);
         }
     }
 
@@ -285,9 +291,10 @@ bool BestPractices::ValidateBindMemory(VkDevice device, VkDeviceMemory memory, c
 
     if (VendorCheckEnabled(kBPVendorNVIDIA) && IsExtEnabled(device_extensions.vk_ext_pageable_device_local_memory)) {
         auto mem_info = std::static_pointer_cast<const bp_state::DeviceMemory>(Get<vvl::DeviceMemory>(memory));
-        if (!mem_info->dynamic_priority) {
+        bool has_static_priority = vku::FindStructInPNextChain<VkMemoryPriorityAllocateInfoEXT>(mem_info->allocate_info.pNext);
+        if (!mem_info->dynamic_priority && !has_static_priority) {
             skip |=
-                LogPerformanceWarning(kVUID_BestPractices_BindMemory_NoPriority, device, loc,
+                LogPerformanceWarning("BestPractices-NVIDIA-BindMemory-NoPriority", device, loc,
                                       "%s Use vkSetDeviceMemoryPriorityEXT to provide the OS with information on which allocations "
                                       "should stay in memory and which should be demoted first when video memory is limited. The "
                                       "highest priority should be given to GPU-written resources like color attachments, depth "
@@ -298,3 +305,33 @@ bool BestPractices::ValidateBindMemory(VkDevice device, VkDeviceMemory memory, c
 
     return skip;
 }
+
+std::shared_ptr<vvl::DeviceMemory> BestPractices::CreateDeviceMemoryState(
+    VkDeviceMemory handle, const VkMemoryAllocateInfo* pAllocateInfo, uint64_t fake_address, const VkMemoryType& memory_type,
+    const VkMemoryHeap& memory_heap, std::optional<vvl::DedicatedBinding>&& dedicated_binding, uint32_t physical_device_count) {
+    return std::static_pointer_cast<vvl::DeviceMemory>(std::make_shared<bp_state::DeviceMemory>(
+        handle, pAllocateInfo, fake_address, memory_type, memory_heap, std::move(dedicated_binding), physical_device_count));
+}
+
+void BestPractices::ManualPostCallRecordBindBufferMemory2(VkDevice device, uint32_t bindInfoCount,
+                                                          const VkBindBufferMemoryInfo* pBindInfos,
+                                                          const RecordObject& record_obj) {
+    if (record_obj.result != VK_SUCCESS && bindInfoCount > 1) {
+        // Details of check found in https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/5527
+        LogWarning(
+            "BestPractices-Partial-Bound-Buffer", device, record_obj.location,
+            "all buffer are now in an indeterminate state because the call failed to return VK_SUCCESS. The best action to take "
+            "is to destroy the buffers instead of trying to rebind");
+    }
+}
+
+void BestPractices::ManualPostCallRecordBindImageMemory2(VkDevice device, uint32_t bindInfoCount,
+                                                         const VkBindImageMemoryInfo* pBindInfos, const RecordObject& record_obj) {
+    if (record_obj.result != VK_SUCCESS && bindInfoCount > 1) {
+        // Details of check found in https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/5527
+        LogWarning(
+            "BestPractices-Partial-Bound-Image", device, record_obj.location,
+            "all image are now in an indeterminate state because the call failed to return VK_SUCCESS. The best action to take is "
+            "to destroy the images instead of trying to rebind");
+    }
+}
