@@ -1,7 +1,7 @@
-/* Copyright (c) 2015-2023 The Khronos Group Inc.
- * Copyright (c) 2015-2023 Valve Corporation
- * Copyright (c) 2015-2023 LunarG, Inc.
- * Copyright (C) 2015-2023 Google Inc.
+/* Copyright (c) 2015-2024 The Khronos Group Inc.
+ * Copyright (c) 2015-2024 Valve Corporation
+ * Copyright (c) 2015-2024 LunarG, Inc.
+ * Copyright (C) 2015-2024 Google Inc.
  * Modifications Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -22,14 +22,19 @@
 #include <fstream>
 #include <vector>
 
-#if defined(__linux__) || defined(__FreeBSD__) || defined(__OpenBSD__)
+#if defined(__linux__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__GNU__)
 #include <unistd.h>
 #endif
 
 #include <vulkan/vk_enum_string_helper.h>
 #include "generated/chassis.h"
 #include "core_validation.h"
-#include "utils/shader_utils.h"
+#include "state_tracker/shader_stage_state.h"
+#include "state_tracker/image_state.h"
+#include "state_tracker/device_state.h"
+#include "state_tracker/buffer_state.h"
+#include "state_tracker/render_pass_state.h"
+#include <spirv-tools/libspirv.h>
 
 bool CoreChecks::ValidateDeviceQueueFamily(uint32_t queue_family, const Location &loc, const char *vuid,
                                            bool optional = false) const {
@@ -80,7 +85,7 @@ bool CoreChecks::ValidatePhysicalDeviceQueueFamilies(uint32_t queue_family_count
 bool CoreChecks::GetPhysicalDeviceImageFormatProperties(vvl::Image &image_state, const char *vuid_string,
                                                         const Location &loc) const {
     bool skip = false;
-    const auto image_create_info = image_state.createInfo;
+    const auto image_create_info = image_state.create_info;
     VkResult image_properties_result = VK_SUCCESS;
     Func command = Func::vkGetPhysicalDeviceImageFormatProperties;
     if (image_create_info.tiling != VK_IMAGE_TILING_DRM_FORMAT_MODIFIER_EXT) {
@@ -97,7 +102,7 @@ bool CoreChecks::GetPhysicalDeviceImageFormatProperties(vvl::Image &image_state,
         image_format_info.flags = image_create_info.flags;
         VkImageFormatProperties2 image_format_properties = vku::InitStructHelper();
         image_properties_result =
-            DispatchGetPhysicalDeviceImageFormatProperties2(physical_device, &image_format_info, &image_format_properties);
+            DispatchGetPhysicalDeviceImageFormatProperties2Helper(physical_device, &image_format_info, &image_format_properties);
         image_state.image_format_properties = image_format_properties.imageFormatProperties;
     }
     if (image_properties_result != VK_SUCCESS) {
@@ -150,31 +155,31 @@ bool CoreChecks::ValidateDeviceMaskToRenderPass(const vvl::CommandBuffer &cb_sta
                                                 const char *vuid) const {
     bool skip = false;
     if ((deviceMask & cb_state.active_render_pass_device_mask) != deviceMask) {
-        skip |= LogError(vuid, cb_state.commandBuffer(), loc, "(0x%" PRIx32 ") is not a subset of %s device mask (0x%" PRIx32 ").",
-                         deviceMask, FormatHandle(cb_state.activeRenderPass->renderPass()).c_str(),
-                         cb_state.active_render_pass_device_mask);
+        skip |=
+            LogError(vuid, cb_state.Handle(), loc, "(0x%" PRIx32 ") is not a subset of %s device mask (0x%" PRIx32 ").", deviceMask,
+                     FormatHandle(cb_state.activeRenderPass->Handle()).c_str(), cb_state.active_render_pass_device_mask);
     }
     return skip;
 }
 
-bool CoreChecks::ValidateQueueFamilyIndex(const vvl::PhysicalDevice *pd_state, uint32_t requested_queue_family, const char *vuid,
+bool CoreChecks::ValidateQueueFamilyIndex(const vvl::PhysicalDevice &pd_state, uint32_t requested_queue_family, const char *vuid,
                                           const Location &loc) const {
     bool skip = false;
 
-    if (requested_queue_family >= pd_state->queue_family_known_count) {
+    if (requested_queue_family >= pd_state.queue_family_known_count) {
         const char *conditional_ext_cmd =
             instance_extensions.vk_khr_get_physical_device_properties2 ? " or vkGetPhysicalDeviceQueueFamilyProperties2[KHR]" : "";
 
-        skip |= LogError(vuid, pd_state->Handle(), loc,
+        skip |= LogError(vuid, pd_state.Handle(), loc,
                          "(%" PRIu32 ") is not less than any previously obtained pQueueFamilyPropertyCount %" PRIu32
                          " from "
                          "vkGetPhysicalDeviceQueueFamilyProperties%s.",
-                         requested_queue_family, pd_state->queue_family_known_count, conditional_ext_cmd);
+                         requested_queue_family, pd_state.queue_family_known_count, conditional_ext_cmd);
     }
     return skip;
 }
 
-bool CoreChecks::ValidateDeviceQueueCreateInfos(const vvl::PhysicalDevice *pd_state, uint32_t info_count,
+bool CoreChecks::ValidateDeviceQueueCreateInfos(const vvl::PhysicalDevice &pd_state, uint32_t info_count,
                                                 const VkDeviceQueueCreateInfo *infos, const Location &loc) const {
     bool skip = false;
 
@@ -204,7 +209,7 @@ bool CoreChecks::ValidateDeviceQueueCreateInfos(const vvl::PhysicalDevice *pd_st
             // Vulkan 1.0 didn't have protected memory so always needed unique info
             create_flags flags = {requested_queue_family, not_used};
             if (queue_family_map.emplace(requested_queue_family, flags).second == false) {
-                skip |= LogError("VUID-VkDeviceCreateInfo-queueFamilyIndex-02802", pd_state->Handle(),
+                skip |= LogError("VUID-VkDeviceCreateInfo-queueFamilyIndex-02802", pd_state.Handle(),
                                  info_loc.dot(Field::queueFamilyIndex),
                                  "(%" PRIu32 ") is not unique and was also used in pCreateInfo->pQueueCreateInfos[%" PRIu32 "].",
                                  requested_queue_family, queue_family_map.at(requested_queue_family).unprocted_index);
@@ -225,7 +230,7 @@ bool CoreChecks::ValidateDeviceQueueCreateInfos(const vvl::PhysicalDevice *pd_st
                 // The queue family was seen, so now need to make sure the flags were different
                 if (protected_create_bit) {
                     if (it->second.protected_index != not_used) {
-                        skip |= LogError("VUID-VkDeviceCreateInfo-queueFamilyIndex-02802", pd_state->Handle(),
+                        skip |= LogError("VUID-VkDeviceCreateInfo-queueFamilyIndex-02802", pd_state.Handle(),
                                          info_loc.dot(Field::queueFamilyIndex),
                                          "(%" PRIu32 ") is not unique and was also used in pCreateInfo->pQueueCreateInfos[%" PRIu32
                                          "] which both have "
@@ -236,7 +241,7 @@ bool CoreChecks::ValidateDeviceQueueCreateInfos(const vvl::PhysicalDevice *pd_st
                     }
                 } else {
                     if (it->second.unprocted_index != not_used) {
-                        skip |= LogError("VUID-VkDeviceCreateInfo-queueFamilyIndex-02802", pd_state->Handle(),
+                        skip |= LogError("VUID-VkDeviceCreateInfo-queueFamilyIndex-02802", pd_state.Handle(),
                                          info_loc.dot(Field::queueFamilyIndex),
                                          "(%" PRIu32 ") is not unique and was also used in pCreateInfo->pQueueCreateInfos[%" PRIu32
                                          "].",
@@ -256,7 +261,7 @@ bool CoreChecks::ValidateDeviceQueueCreateInfos(const vvl::PhysicalDevice *pd_st
         const auto prev_global_priority = global_priorities.find(infos[i].queueFamilyIndex);
         if (prev_global_priority != global_priorities.end()) {
             if (prev_global_priority->second != global_priority) {
-                skip |= LogError("VUID-VkDeviceCreateInfo-pQueueCreateInfos-06654", pd_state->Handle(), info_loc,
+                skip |= LogError("VUID-VkDeviceCreateInfo-pQueueCreateInfos-06654", pd_state.Handle(), info_loc,
                                  "Multiple queues are created with queueFamilyIndex %" PRIu32
                                  ", but one has global priority %s and another %s.",
                                  infos[i].queueFamilyIndex, string_VkQueueGlobalPriorityKHR(prev_global_priority->second),
@@ -266,20 +271,20 @@ bool CoreChecks::ValidateDeviceQueueCreateInfos(const vvl::PhysicalDevice *pd_st
             global_priorities.insert({infos[i].queueFamilyIndex, global_priority});
         }
 
-        const VkQueueFamilyProperties requested_queue_family_props = pd_state->queue_family_properties[requested_queue_family];
+        const VkQueueFamilyProperties requested_queue_family_props = pd_state.queue_family_properties[requested_queue_family];
 
         // if using protected flag, make sure queue supports it
         if (protected_create_bit && ((requested_queue_family_props.queueFlags & VK_QUEUE_PROTECTED_BIT) == 0)) {
-            skip |= LogError("VUID-VkDeviceQueueCreateInfo-flags-06449", pd_state->Handle(), info_loc.dot(Field::queueFamilyIndex),
+            skip |= LogError("VUID-VkDeviceQueueCreateInfo-flags-06449", pd_state.Handle(), info_loc.dot(Field::queueFamilyIndex),
                              "(%" PRIu32 ") does not have VK_QUEUE_PROTECTED_BIT supported, but pQueueCreateInfos[%" PRIu32
                              "].flags has VK_DEVICE_QUEUE_CREATE_PROTECTED_BIT.",
                              requested_queue_family, i);
         }
 
         // Verify that requested queue count of queue family is known to be valid at this point in time
-        if (requested_queue_family < pd_state->queue_family_known_count) {
+        if (requested_queue_family < pd_state.queue_family_known_count) {
             const auto requested_queue_count = infos[i].queueCount;
-            const bool queue_family_has_props = requested_queue_family < pd_state->queue_family_properties.size();
+            const bool queue_family_has_props = requested_queue_family < pd_state.queue_family_properties.size();
             // spec guarantees at least one queue for each queue family
             const uint32_t available_queue_count = queue_family_has_props ? requested_queue_family_props.queueCount : 1;
 
@@ -293,7 +298,7 @@ bool CoreChecks::ValidateDeviceQueueCreateInfos(const vvl::PhysicalDevice *pd_st
                         : "the pQueueFamilyProperties[" + std::to_string(requested_queue_family) + "] was never obtained";
 
                 skip |= LogError(
-                    "VUID-VkDeviceQueueCreateInfo-queueCount-00382", pd_state->Handle(), info_loc.dot(Field::queueCount),
+                    "VUID-VkDeviceQueueCreateInfo-queueCount-00382", pd_state.Handle(), info_loc.dot(Field::queueCount),
                     " (%" PRIu32
                     ") is not less than or equal to available queue count for this pCreateInfo->pQueueCreateInfos[%" PRIu32
                     "].queueFamilyIndex} (%" PRIu32 ") obtained previously from vkGetPhysicalDeviceQueueFamilyProperties%s (%s).",
@@ -307,11 +312,11 @@ bool CoreChecks::ValidateDeviceQueueCreateInfos(const vvl::PhysicalDevice *pd_st
         }
     }
     for (uint32_t i = 0; i < static_cast<uint32_t>(queue_counts.size()); ++i) {
-        if (queue_counts[i] > pd_state->queue_family_properties[i].queueCount) {
-            skip |= LogError("VUID-VkDeviceCreateInfo-pQueueCreateInfos-06755", pd_state->Handle(), loc,
+        if (queue_counts[i] > pd_state.queue_family_properties[i].queueCount) {
+            skip |= LogError("VUID-VkDeviceCreateInfo-pQueueCreateInfos-06755", pd_state.Handle(), loc,
                              "Total queue count requested from queue family index %" PRIu32 " is %" PRIu32
                              ", which is greater than queue count available in the queue family (%" PRIu32 ").",
-                             i, queue_counts[i], pd_state->queue_family_properties[i].queueCount);
+                             i, queue_counts[i], pd_state.queue_family_properties[i].queueCount);
         }
     }
 
@@ -330,7 +335,7 @@ bool CoreChecks::PreCallValidateCreateDevice(VkPhysicalDevice gpu, const VkDevic
         skip |= LogError("VUID-vkCreateDevice-physicalDevice-parameter", device, error_obj.location,
                          "Have not called vkEnumeratePhysicalDevices() yet.");
     } else {
-        skip |= ValidateDeviceQueueCreateInfos(pd_state.get(), pCreateInfo->queueCreateInfoCount, pCreateInfo->pQueueCreateInfos,
+        skip |= ValidateDeviceQueueCreateInfos(*pd_state, pCreateInfo->queueCreateInfoCount, pCreateInfo->pQueueCreateInfos,
                                                error_obj.location.dot(Field::pCreateInfo));
 
         const VkPhysicalDeviceFragmentShadingRateFeaturesKHR *fragment_shading_rate_features =
@@ -432,9 +437,9 @@ bool CoreChecks::PreCallValidateCreateDevice(VkPhysicalDevice gpu, const VkDevic
     return skip;
 }
 
-void CoreChecks::CreateDevice(const VkDeviceCreateInfo *pCreateInfo) {
-    // The state tracker sets up the device state
-    StateTracker::CreateDevice(pCreateInfo);
+void CoreChecks::PostCreateDevice(const VkDeviceCreateInfo *pCreateInfo, const Location &loc) {
+    // The state tracker sets up the device state (also if extension and/or features are enabled)
+    StateTracker::PostCreateDevice(pCreateInfo, loc);
 
     // Add the callback hooks for the functions that are either broadly or deeply used and that the ValidationStateTracker refactor
     // would be messier without.
@@ -444,11 +449,13 @@ void CoreChecks::CreateDevice(const VkDeviceCreateInfo *pCreateInfo) {
             cb_state->SetImageViewInitialLayout(iv_state, layout);
         });
 
+    AdjustValidatorOptions(device_extensions, enabled_features, spirv_val_options, &spirv_val_option_hash);
+
     // Allocate shader validation cache
     if (!disabled[shader_validation_caching] && !disabled[shader_validation] && !core_validation_cache) {
         auto tmp_path = GetTempFilePath();
         validation_cache_path = tmp_path + "/shader_validation_cache";
-#if defined(__linux__) || defined(__FreeBSD__) || defined(__OpenBSD__)
+#if defined(__linux__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__GNU__)
         validation_cache_path += "-" + std::to_string(getuid());
 #endif
         validation_cache_path += ".bin";
@@ -460,7 +467,6 @@ void CoreChecks::CreateDevice(const VkDeviceCreateInfo *pCreateInfo) {
             std::copy(std::istreambuf_iterator<char>(read_file), {}, std::back_inserter(validation_cache_data));
             read_file.close();
         } else {
-            Location loc(Func::vkCreateDevice);
             LogInfo("WARNING-cache-file-error", device, loc,
                     "Cannot open shader validation cache at %s for reading (it may not exist yet)", validation_cache_path.c_str());
         }
@@ -540,7 +546,7 @@ bool CoreChecks::PreCallValidateGetDeviceQueue(VkDevice device, uint32_t queueFa
         }
 
         if (device_queue_info.queue_count <= queueIndex) {
-            skip |= LogError("VUID-vkGetDeviceQueue-queueIndex-00385", device, error_obj.location.dot(Field::queueFamilyIndex),
+            skip |= LogError("VUID-vkGetDeviceQueue-queueIndex-00385", device, error_obj.location.dot(Field::queueIndex),
                              "(%" PRIu32 ") is not less than the number of queues requested from queueFamilyIndex (=%" PRIu32
                              ") when the device was created vkCreateDevice::pCreateInfo->pQueueCreateInfos[%" PRIu32
                              "] (i.e. is not less than %" PRIu32 ").",
@@ -620,7 +626,8 @@ bool CoreChecks::PreCallValidateGetPhysicalDeviceImageFormatProperties2(VkPhysic
                                                                         VkImageFormatProperties2 *pImageFormatProperties,
                                                                         const ErrorObject &error_obj) const {
     // Can't wrap AHB-specific validation in a device extension check here, but no harm
-    bool skip = ValidateGetPhysicalDeviceImageFormatProperties2ANDROID(pImageFormatInfo, pImageFormatProperties, error_obj);
+    bool skip = false;
+    skip |= ValidateGetPhysicalDeviceImageFormatProperties2ANDROID(pImageFormatInfo, pImageFormatProperties, error_obj);
     skip |= ValidateGetPhysicalDeviceImageFormatProperties2(pImageFormatInfo, pImageFormatProperties, error_obj);
     return skip;
 }
@@ -639,7 +646,7 @@ VkFormatProperties3KHR CoreChecks::GetPDFormatProperties(const VkFormat format)
     VkFormatProperties2 fmt_props_2 = vku::InitStructHelper(&fmt_props_3);
 
     if (has_format_feature2) {
-        DispatchGetPhysicalDeviceFormatProperties2(physical_device, format, &fmt_props_2);
+        DispatchGetPhysicalDeviceFormatProperties2Helper(physical_device, format, &fmt_props_2);
         fmt_props_3.linearTilingFeatures |= fmt_props_2.formatProperties.linearTilingFeatures;
         fmt_props_3.optimalTilingFeatures |= fmt_props_2.formatProperties.optimalTilingFeatures;
         fmt_props_3.bufferFeatures |= fmt_props_2.formatProperties.bufferFeatures;
@@ -656,7 +663,7 @@ VkFormatProperties3KHR CoreChecks::GetPDFormatProperties(const VkFormat format)
 VkResult CoreChecks::CoreLayerCreateValidationCacheEXT(VkDevice device, const VkValidationCacheCreateInfoEXT *pCreateInfo,
                                                        const VkAllocationCallbacks *pAllocator,
                                                        VkValidationCacheEXT *pValidationCache) {
-    *pValidationCache = ValidationCache::Create(pCreateInfo);
+    *pValidationCache = ValidationCache::Create(pCreateInfo, spirv_val_option_hash);
     return *pValidationCache ? VK_SUCCESS : VK_ERROR_INITIALIZATION_FAILED;
 }
 
@@ -754,8 +761,10 @@ bool CoreChecks::PreCallValidateCreateCommandPool(VkDevice device, const VkComma
 bool CoreChecks::PreCallValidateDestroyCommandPool(VkDevice device, VkCommandPool commandPool,
                                                    const VkAllocationCallbacks *pAllocator, const ErrorObject &error_obj) const {
     bool skip = false;
+    // In case of DEVICE_LOST, all execution is considered over
+    if (is_device_lost) return skip;
     auto cp_state = Get<vvl::CommandPool>(commandPool);
-    if (!cp_state) { return false; }
+    if (!cp_state) return skip;
     // Verify that command buffers in pool are complete (not in-flight)
     for (auto &entry : cp_state->commandBuffers) {
         auto cb_state = entry.second;
@@ -772,7 +781,7 @@ bool CoreChecks::PreCallValidateResetCommandPool(VkDevice device, VkCommandPool
                                                  const ErrorObject &error_obj) const {
     bool skip = false;
     auto cp_state = Get<vvl::CommandPool>(commandPool);
-    if (!cp_state) { return false; }
+    ASSERT_AND_RETURN_SKIP(cp_state);
     // Verify that command buffers in pool are complete (not in-flight)
     for (auto &entry : cp_state->commandBuffers) {
         auto cb_state = entry.second;
@@ -788,6 +797,8 @@ bool CoreChecks::PreCallValidateResetCommandPool(VkDevice device, VkCommandPool
 // For given obj node, if it is use, flag a validation error and return callback result, else return false
 bool CoreChecks::ValidateObjectNotInUse(const vvl::StateObject *obj_node, const Location &loc, const char *error_code) const {
     if (disabled[object_in_use]) return false;
+    // In case of DEVICE_LOST, all execution is considered over
+    if (is_device_lost) return false;
     auto obj_struct = obj_node->Handle();
     bool skip = false;
 
@@ -813,7 +824,7 @@ bool CoreChecks::PreCallValidateGetCalibratedTimestampsKHR(VkDevice device, uint
                                                            const ErrorObject &error_obj) const {
     bool skip = false;
 
-    auto query_function = (error_obj.location.function == Func::vkGetPhysicalDeviceCalibrateableTimeDomainsKHR)
+    auto query_function = (error_obj.location.function == Func::vkGetCalibratedTimestampsKHR)
                               ? DispatchGetPhysicalDeviceCalibrateableTimeDomainsKHR
                               : DispatchGetPhysicalDeviceCalibrateableTimeDomainsEXT;
     uint32_t count = 0;
@@ -840,3 +851,96 @@ bool CoreChecks::PreCallValidateGetCalibratedTimestampsKHR(VkDevice device, uint
     }
     return skip;
 }
+
+// These were all added from https://gitlab.khronos.org/vulkan/vulkan/-/merge_requests/6672
+bool CoreChecks::ValidateDeviceQueueSupport(const Location &loc) const {
+    bool skip = false;
+    const char *vuid = kVUIDUndefined;
+    VkQueueFlags flags = 0;
+
+    switch (loc.function) {
+        case Func::vkCreateRayTracingPipelinesKHR:
+            vuid = "VUID-vkCreateRayTracingPipelinesKHR-device-09677";
+            flags = VK_QUEUE_COMPUTE_BIT;
+            break;
+        case Func::vkCreateRayTracingPipelinesNV:
+            vuid = "VUID-vkCreateRayTracingPipelinesNV-device-09677";
+            flags = VK_QUEUE_COMPUTE_BIT;
+            break;
+        case Func::vkCreateComputePipelines:
+            vuid = "VUID-vkCreateComputePipelines-device-09661";
+            flags = VK_QUEUE_COMPUTE_BIT;
+            break;
+        case Func::vkCreateGraphicsPipelines:
+            vuid = "VUID-vkCreateGraphicsPipelines-device-09662";
+            flags = VK_QUEUE_GRAPHICS_BIT;
+            break;
+        case Func::vkCreateQueryPool:
+            vuid = "VUID-vkCreateQueryPool-device-09663";
+            flags = VK_QUEUE_VIDEO_ENCODE_BIT_KHR | VK_QUEUE_VIDEO_DECODE_BIT_KHR | VK_QUEUE_COMPUTE_BIT | VK_QUEUE_GRAPHICS_BIT;
+            break;
+        case Func::vkCreateBuffer:
+            vuid = "VUID-vkCreateBuffer-device-09664";
+            flags = VK_QUEUE_VIDEO_ENCODE_BIT_KHR | VK_QUEUE_VIDEO_DECODE_BIT_KHR | VK_QUEUE_SPARSE_BINDING_BIT |
+                    VK_QUEUE_TRANSFER_BIT | VK_QUEUE_COMPUTE_BIT | VK_QUEUE_GRAPHICS_BIT;
+            break;
+        case Func::vkCreateBufferView:
+            vuid = "VUID-vkCreateBufferView-device-09665";
+            flags = VK_QUEUE_COMPUTE_BIT | VK_QUEUE_GRAPHICS_BIT;
+            break;
+        case Func::vkCreateImage:
+            vuid = "VUID-vkCreateImage-device-09666";
+            flags = VK_QUEUE_VIDEO_ENCODE_BIT_KHR | VK_QUEUE_VIDEO_DECODE_BIT_KHR | VK_QUEUE_OPTICAL_FLOW_BIT_NV |
+                    VK_QUEUE_SPARSE_BINDING_BIT | VK_QUEUE_TRANSFER_BIT | VK_QUEUE_COMPUTE_BIT | VK_QUEUE_GRAPHICS_BIT;
+            break;
+        case Func::vkCreateImageView:
+            vuid = "VUID-vkCreateImageView-device-09667";
+            flags = VK_QUEUE_VIDEO_ENCODE_BIT_KHR | VK_QUEUE_VIDEO_DECODE_BIT_KHR | VK_QUEUE_COMPUTE_BIT | VK_QUEUE_GRAPHICS_BIT;
+            break;
+        case Func::vkCreateSampler:
+            vuid = "VUID-vkCreateSampler-device-09668";
+            flags = VK_QUEUE_COMPUTE_BIT | VK_QUEUE_GRAPHICS_BIT;
+            break;
+        case Func::vkCreateEvent:
+            vuid = "VUID-vkCreateEvent-device-09672";
+            flags = VK_QUEUE_VIDEO_ENCODE_BIT_KHR | VK_QUEUE_VIDEO_DECODE_BIT_KHR | VK_QUEUE_COMPUTE_BIT | VK_QUEUE_GRAPHICS_BIT;
+            break;
+        case Func::vkCreateShadersEXT:
+            vuid = "VUID-vkCreateShadersEXT-device-09669";
+            flags = VK_QUEUE_COMPUTE_BIT | VK_QUEUE_GRAPHICS_BIT;
+            break;
+        case Func::vkCreateRenderPass:
+            vuid = "VUID-vkCreateRenderPass-device-10000";
+            flags = VK_QUEUE_GRAPHICS_BIT;
+            break;
+        case Func::vkCreateRenderPass2:
+        case Func::vkCreateRenderPass2KHR:
+            vuid = "VUID-vkCreateRenderPass2-device-10001";
+            flags = VK_QUEUE_GRAPHICS_BIT;
+            break;
+        case Func::vkCreateFramebuffer:
+            vuid = "VUID-vkCreateFramebuffer-device-10002";
+            flags = VK_QUEUE_GRAPHICS_BIT;
+            break;
+        default:
+            assert(false);  // missing case
+            return skip;
+    }
+
+    if ((physical_device_state->supported_queues & flags) == 0) {
+        skip |= LogError(vuid, device, loc, "device only supports (%s) but require one of (%s).",
+                         string_VkQueueFlags(physical_device_state->supported_queues).c_str(), string_VkQueueFlags(flags).c_str());
+    }
+
+    return skip;
+}
+
+bool CoreChecks::PreCallValidateGetDeviceFaultInfoEXT(VkDevice device, VkDeviceFaultCountsEXT *pFaultCounts,
+                                                      VkDeviceFaultInfoEXT *pFaultInfo, const ErrorObject &error_obj) const {
+    bool skip = false;
+    if (!is_device_lost) {
+        skip |= LogError("VUID-vkGetDeviceFaultInfoEXT-device-07336", device, error_obj.location,
+                         "device has not been found to be in a lost state.");
+    }
+    return skip;
+}
