@@ -1,7 +1,7 @@
-/* Copyright (c) 2015-2023 The Khronos Group Inc.
- * Copyright (c) 2015-2023 Valve Corporation
- * Copyright (c) 2015-2023 LunarG, Inc.
- * Copyright (C) 2015-2023 Google Inc.
+/* Copyright (c) 2015-2024 The Khronos Group Inc.
+ * Copyright (c) 2015-2024 Valve Corporation
+ * Copyright (c) 2015-2024 LunarG, Inc.
+ * Copyright (C) 2015-2024 Google Inc.
  * Modifications Copyright (C) 2020 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -19,9 +19,7 @@
 #pragma once
 
 #include "vulkan/vulkan.h"
-#include "generated/vk_object_types.h"
 #include "containers/custom_containers.h"
-#include "error_message/logging.h"
 #include "utils/vk_layer_utils.h"
 
 #include <atomic>
@@ -49,9 +47,10 @@ class StateObject: public std::enable_shared_from_this<StateObject>, public Type
     // parent types without locking every weak_ptr.
     using NodeMap = unordered_map<VulkanTypedHandle, std::weak_ptr<StateObject>>;
     using NodeList = small_vector<std::shared_ptr<StateObject>, 4, uint32_t>;
+    using IdType = uint32_t;
 
     template <typename Handle>
-    StateObject(Handle h, VulkanObjectType t) : TypedHandleWrapper(h, t), destroyed_(false) {}
+    StateObject(Handle h, VulkanObjectType t) : TypedHandleWrapper(h, t), destroyed_(false), id_(0) {}
 
     // because shared_from_this() does not work from the constructor, this 2nd phase
     // constructor is where a state object should call AddParent() on its child nodes.
@@ -68,6 +67,12 @@ class StateObject: public std::enable_shared_from_this<StateObject>, public Type
 
     bool Destroyed() const { return destroyed_; }
 
+    // Some drivers may reuse vulkan handles, which can confuse some parts of validation that cache state.
+    // Add a unique id to help detect this condition. SetId() should only be called by the state tracker during
+    // object creation.
+    void SetId(IdType id) { id_ = id; }
+    IdType GetId() const { return id_; }
+
     // returns true if this vulkan object or any it uses have been destroyed
     virtual bool Invalid() const { return Destroyed(); }
 
@@ -109,7 +114,7 @@ class StateObject: public std::enable_shared_from_this<StateObject>, public Type
     // Set to true when the API-level object is destroyed, but this object may
     // hang around until its shared_ptr refcount goes to zero.
     std::atomic<bool> destroyed_;
-
+    IdType id_;
   private:
     ReadLockGuard ReadLockTree() const { return ReadLockGuard(tree_lock_); }
     WriteLockGuard WriteLockTree() { return WriteLockGuard(tree_lock_); }
@@ -128,7 +133,7 @@ class RefcountedStateObject : public StateObject {
 
   public:
     template <typename Handle>
-    RefcountedStateObject(Handle h, VulkanObjectType t) : StateObject(h, t), in_use_(0) {}
+    RefcountedStateObject(Handle handle, VulkanObjectType type) : StateObject(handle, type), in_use_(0) {}
 
     void BeginUse() { in_use_.fetch_add(1); }
 
