@@ -1,7 +1,7 @@
-/* Copyright (c) 2015-2023 The Khronos Group Inc.
- * Copyright (c) 2015-2023 Valve Corporation
- * Copyright (c) 2015-2023 LunarG, Inc.
- * Copyright (C) 2015-2023 Google Inc.
+/* Copyright (c) 2015-2024 The Khronos Group Inc.
+ * Copyright (c) 2015-2024 Valve Corporation
+ * Copyright (c) 2015-2024 LunarG, Inc.
+ * Copyright (C) 2015-2024 Google Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,48 +22,46 @@
 ReadLockGuard StatelessValidation::ReadLock() const { return ReadLockGuard(validation_object_mutex, std::defer_lock); }
 WriteLockGuard StatelessValidation::WriteLock() { return WriteLockGuard(validation_object_mutex, std::defer_lock); }
 
-static vvl::unordered_map<VkCommandBuffer, VkCommandPool> secondary_cb_map{};
-static std::shared_mutex secondary_cb_map_mutex;
-static ReadLockGuard CBReadLock() { return ReadLockGuard(secondary_cb_map_mutex); }
-static WriteLockGuard CBWriteLock() { return WriteLockGuard(secondary_cb_map_mutex); }
-
-bool StatelessValidation::manual_PreCallValidateCmdBindIndexBuffer(VkCommandBuffer commandBuffer, VkBuffer buffer,
-                                                                   VkDeviceSize offset, VkIndexType indexType,
-                                                                   const ErrorObject &error_obj) const {
+bool StatelessValidation::ValidateCmdBindIndexBuffer(VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
+                                                     VkIndexType indexType, const Location &loc) const {
     bool skip = false;
+    const bool is_2 = loc.function != Func::vkCmdBindIndexBuffer;
+    const char *vuid;
 
-    if (indexType == VK_INDEX_TYPE_NONE_KHR) {
-        skip |= LogError("VUID-vkCmdBindIndexBuffer-indexType-08786", commandBuffer, error_obj.location.dot(Field::indexType),
-                         "is VK_INDEX_TYPE_NONE_KHR.");
+    if (buffer == VK_NULL_HANDLE) {
+        if (!enabled_features.maintenance6) {
+            vuid = is_2 ? "VUID-vkCmdBindIndexBuffer2KHR-None-09493" : "VUID-vkCmdBindIndexBuffer-None-09493";
+            skip |= LogError(vuid, commandBuffer, loc.dot(Field::buffer), "is VK_NULL_HANDLE.");
+        } else if (offset != 0) {
+            vuid = is_2 ? "VUID-vkCmdBindIndexBuffer2KHR-buffer-09494" : "VUID-vkCmdBindIndexBuffer-buffer-09494";
+            skip |= LogError(vuid, commandBuffer, loc.dot(Field::buffer), "is VK_NULL_HANDLE but offset is (%" PRIu64 ").", offset);
+        }
     }
 
-    const auto *index_type_uint8_features = vku::FindStructInPNextChain<VkPhysicalDeviceIndexTypeUint8FeaturesEXT>(device_createinfo_pnext);
-    if (indexType == VK_INDEX_TYPE_UINT8_EXT && (!index_type_uint8_features || !index_type_uint8_features->indexTypeUint8)) {
-        skip |= LogError("VUID-vkCmdBindIndexBuffer-indexType-08787", commandBuffer, error_obj.location.dot(Field::indexType),
-                         "is VK_INDEX_TYPE_UINT8_EXT but indexTypeUint8 feature was not enabled.");
+    if (indexType == VK_INDEX_TYPE_NONE_KHR) {
+        vuid = is_2 ? "VUID-vkCmdBindIndexBuffer2KHR-indexType-08786" : "VUID-vkCmdBindIndexBuffer-indexType-08786";
+        skip |= LogError(vuid, commandBuffer, loc.dot(Field::indexType), "is VK_INDEX_TYPE_NONE_KHR.");
+    }
+
+    if (indexType == VK_INDEX_TYPE_UINT8_KHR && !enabled_features.indexTypeUint8) {
+        vuid = is_2 ? "VUID-vkCmdBindIndexBuffer2KHR-indexType-08787" : "VUID-vkCmdBindIndexBuffer-indexType-08787";
+        skip |= LogError(vuid, commandBuffer, loc.dot(Field::indexType),
+                         "is VK_INDEX_TYPE_UINT8_KHR but indexTypeUint8 feature was not enabled.");
     }
 
     return skip;
 }
 
+bool StatelessValidation::manual_PreCallValidateCmdBindIndexBuffer(VkCommandBuffer commandBuffer, VkBuffer buffer,
+                                                                   VkDeviceSize offset, VkIndexType indexType,
+                                                                   const ErrorObject &error_obj) const {
+    return ValidateCmdBindIndexBuffer(commandBuffer, buffer, offset, indexType, error_obj.location);
+}
+
 bool StatelessValidation::manual_PreCallValidateCmdBindIndexBuffer2KHR(VkCommandBuffer commandBuffer, VkBuffer buffer,
                                                                        VkDeviceSize offset, VkDeviceSize size,
                                                                        VkIndexType indexType, const ErrorObject &error_obj) const {
-    bool skip = false;
-
-    if (indexType == VK_INDEX_TYPE_NONE_KHR) {
-        skip |= LogError("VUID-vkCmdBindIndexBuffer2KHR-indexType-08786", commandBuffer, error_obj.location.dot(Field::indexType),
-                         "is VK_INDEX_TYPE_NONE_KHR.");
-    } else if (indexType == VK_INDEX_TYPE_UINT8_EXT) {
-        const auto *index_type_uint8_features = vku::FindStructInPNextChain<VkPhysicalDeviceIndexTypeUint8FeaturesEXT>(device_createinfo_pnext);
-        if (!index_type_uint8_features || !index_type_uint8_features->indexTypeUint8) {
-            skip |=
-                LogError("VUID-vkCmdBindIndexBuffer2KHR-indexType-08787", commandBuffer, error_obj.location.dot(Field::indexType),
-                         "is VK_INDEX_TYPE_UINT8_EXT but indexTypeUint8 feature was not enabled.");
-        }
-    }
-
-    return skip;
+    return ValidateCmdBindIndexBuffer(commandBuffer, buffer, offset, indexType, error_obj.location);
 }
 
 bool StatelessValidation::manual_PreCallValidateCmdBindVertexBuffers(VkCommandBuffer commandBuffer, uint32_t firstBinding,
@@ -84,10 +82,14 @@ bool StatelessValidation::manual_PreCallValidateCmdBindVertexBuffers(VkCommandBu
     }
 
     for (uint32_t i = 0; i < bindingCount; ++i) {
+        if (pBuffers == nullptr) {
+            skip |= LogError("VUID-vkCmdBindVertexBuffers-pBuffers-parameter", commandBuffer,
+                             error_obj.location.dot(Field::pBuffers), "is NULL.");
+            return skip;
+        }
         if (pBuffers[i] == VK_NULL_HANDLE) {
             const Location buffer_loc = error_obj.location.dot(Field::pBuffers, i);
-            const auto *robustness2_features = vku::FindStructInPNextChain<VkPhysicalDeviceRobustness2FeaturesEXT>(device_createinfo_pnext);
-            if (!(robustness2_features && robustness2_features->nullDescriptor)) {
+            if (!enabled_features.nullDescriptor) {
                 skip |= LogError("VUID-vkCmdBindVertexBuffers-pBuffers-04001", commandBuffer, buffer_loc, "is VK_NULL_HANDLE.");
             } else {
                 if (pOffsets[i] != 0) {
@@ -106,6 +108,11 @@ bool StatelessValidation::manual_PreCallValidateCmdBindTransformFeedbackBuffersE
     const VkDeviceSize *pOffsets, const VkDeviceSize *pSizes, const ErrorObject &error_obj) const {
     bool skip = false;
 
+    if (!enabled_features.transformFeedback) {
+        skip |= LogError("VUID-vkCmdBindTransformFeedbackBuffersEXT-transformFeedback-02355", commandBuffer, error_obj.location,
+                         "transformFeedback feature was not enabled.");
+    }
+
     for (uint32_t i = 0; i < bindingCount; ++i) {
         if (pOffsets[i] & 3) {
             skip |= LogError("VUID-vkCmdBindTransformFeedbackBuffersEXT-pOffsets-02359", commandBuffer,
@@ -150,6 +157,10 @@ bool StatelessValidation::manual_PreCallValidateCmdBeginTransformFeedbackEXT(
     VkCommandBuffer commandBuffer, uint32_t firstCounterBuffer, uint32_t counterBufferCount, const VkBuffer *pCounterBuffers,
     const VkDeviceSize *pCounterBufferOffsets, const ErrorObject &error_obj) const {
     bool skip = false;
+    if (!enabled_features.transformFeedback) {
+        skip |= LogError("VUID-vkCmdBeginTransformFeedbackEXT-transformFeedback-02366", commandBuffer, error_obj.location,
+                         "transformFeedback feature was not enabled.");
+    }
 
     if (firstCounterBuffer >= phys_dev_ext_props.transform_feedback_props.maxTransformFeedbackBuffers) {
         skip |= LogError("VUID-vkCmdBeginTransformFeedbackEXT-firstCounterBuffer-02368", commandBuffer, error_obj.location,
@@ -176,6 +187,17 @@ bool StatelessValidation::manual_PreCallValidateCmdEndTransformFeedbackEXT(VkCom
                                                                            const VkDeviceSize *pCounterBufferOffsets,
                                                                            const ErrorObject &error_obj) const {
     bool skip = false;
+    if (!enabled_features.transformFeedback) {
+        skip |= LogError("VUID-vkCmdEndTransformFeedbackEXT-transformFeedback-02374", commandBuffer, error_obj.location,
+                         "transformFeedback feature was not enabled.");
+    }
+
+    // pCounterBuffers and pCounterBufferOffsets are optional and may be nullptr.
+    //  Additionaly, pCounterBufferOffsets must be nullptr if pCounterBuffers is nullptr.
+    if (pCounterBuffers == nullptr && pCounterBufferOffsets != nullptr) {
+        skip |= LogError("VUID-vkCmdEndTransformFeedbackEXT-pCounterBuffer-02379", commandBuffer, error_obj.location,
+                         "pCounterBuffers is NULL and pCounterBufferOffsets is not NULL.");
+    }
 
     if (firstCounterBuffer >= phys_dev_ext_props.transform_feedback_props.maxTransformFeedbackBuffers) {
         skip |= LogError("VUID-vkCmdEndTransformFeedbackEXT-firstCounterBuffer-02376", commandBuffer, error_obj.location,
@@ -204,6 +226,7 @@ bool StatelessValidation::manual_PreCallValidateCmdBindVertexBuffers2(VkCommandB
     bool skip = false;
 
     // Check VUID-vkCmdBindVertexBuffers2-bindingCount-arraylength
+    // This is a special case and generator currently skips it
     {
         const bool vuidCondition = (pSizes != nullptr) || (pStrides != nullptr);
         const bool vuidExpectation = bindingCount > 0;
@@ -220,31 +243,40 @@ bool StatelessValidation::manual_PreCallValidateCmdBindVertexBuffers2(VkCommandB
                                  "%s, so bindingCount must be greater than 0.", not_null_msg);
             }
         }
+
+        if (!pOffsets && bindingCount > 0) {
+            skip |= LogError("VUID-vkCmdBindVertexBuffers2-pOffsets-parameter", commandBuffer,
+                             error_obj.location.dot(Field::pOffsets), "is NULL.");
+        }
     }
 
     if (firstBinding >= device_limits.maxVertexInputBindings) {
-        skip |= LogError("VUID-vkCmdBindVertexBuffers2-firstBinding-03355", commandBuffer, error_obj.location,
-                         "firstBinding (%" PRIu32 ") must be less than maxVertexInputBindings (%" PRIu32 ").", firstBinding,
-                         device_limits.maxVertexInputBindings);
+        skip |=
+            LogError("VUID-vkCmdBindVertexBuffers2-firstBinding-03355", commandBuffer, error_obj.location.dot(Field::firstBinding),
+                     "(%" PRIu32 ") must be less than maxVertexInputBindings (%" PRIu32 ").", firstBinding,
+                     device_limits.maxVertexInputBindings);
     } else if ((firstBinding + bindingCount) > device_limits.maxVertexInputBindings) {
-        skip |= LogError("VUID-vkCmdBindVertexBuffers2-firstBinding-03356", commandBuffer, error_obj.location,
-                         "sum of firstBinding (%" PRIu32 ") and bindingCount (%" PRIu32
-                         ") must be less than "
-                         "maxVertexInputBindings (%" PRIu32 ").",
-                         firstBinding, bindingCount, device_limits.maxVertexInputBindings);
+        skip |=
+            LogError("VUID-vkCmdBindVertexBuffers2-firstBinding-03356", commandBuffer, error_obj.location.dot(Field::firstBinding),
+                     "(%" PRIu32 ") + bindingCount (%" PRIu32
+                     ") must be less than "
+                     "maxVertexInputBindings (%" PRIu32 ").",
+                     firstBinding, bindingCount, device_limits.maxVertexInputBindings);
     }
 
     for (uint32_t i = 0; i < bindingCount; ++i) {
+        if (pBuffers == nullptr) {
+            skip |= LogError("VUID-vkCmdBindVertexBuffers2-pBuffers-parameter", commandBuffer,
+                             error_obj.location.dot(Field::pBuffers), "is NULL.");
+            return skip;
+        }
         if (pBuffers[i] == VK_NULL_HANDLE) {
             const Location buffer_loc = error_obj.location.dot(Field::pBuffers, i);
-            const auto *robustness2_features = vku::FindStructInPNextChain<VkPhysicalDeviceRobustness2FeaturesEXT>(device_createinfo_pnext);
-            if (!(robustness2_features && robustness2_features->nullDescriptor)) {
+            if (!enabled_features.nullDescriptor) {
                 skip |= LogError("VUID-vkCmdBindVertexBuffers2-pBuffers-04111", commandBuffer, buffer_loc, "is VK_NULL_HANDLE.");
-            } else {
-                if (pOffsets[i] != 0) {
-                    skip |= LogError("VUID-vkCmdBindVertexBuffers2-pBuffers-04112", commandBuffer, buffer_loc,
-                                     "is VK_NULL_HANDLE, but pOffsets[%" PRIu32 "] is not 0.", i);
-                }
+            } else if (pOffsets && pOffsets[i] != 0) {
+                skip |= LogError("VUID-vkCmdBindVertexBuffers2-pBuffers-04112", commandBuffer, buffer_loc,
+                                 "is VK_NULL_HANDLE, but pOffsets[%" PRIu32 "] is not 0.", i);
             }
         }
         if (pStrides) {
@@ -301,8 +333,20 @@ bool StatelessValidation::manual_PreCallValidateCmdPushConstants(VkCommandBuffer
 bool StatelessValidation::manual_PreCallValidateCmdPushConstants2KHR(VkCommandBuffer commandBuffer,
                                                                      const VkPushConstantsInfoKHR *pPushConstantsInfo,
                                                                      const ErrorObject &error_obj) const {
-    return ValidateCmdPushConstants(commandBuffer, pPushConstantsInfo->offset, pPushConstantsInfo->size,
-                                    error_obj.location.dot(Field::pPushConstantsInfo));
+    bool skip = false;
+    skip |= ValidateCmdPushConstants(commandBuffer, pPushConstantsInfo->offset, pPushConstantsInfo->size,
+                                     error_obj.location.dot(Field::pPushConstantsInfo));
+    if (pPushConstantsInfo->layout == VK_NULL_HANDLE) {
+        if (!enabled_features.dynamicPipelineLayout) {
+            skip |= LogError("VUID-VkPushConstantsInfoKHR-None-09495", commandBuffer,
+                             error_obj.location.dot(Field::pPushConstantsInfo).dot(Field::layout), "is VK_NULL_HANDLE.");
+        } else if (!vku::FindStructInPNextChain<VkPipelineLayoutCreateInfo>(pPushConstantsInfo->pNext)) {
+            skip |= LogError("VUID-VkPushConstantsInfoKHR-layout-09496", commandBuffer,
+                             error_obj.location.dot(Field::pPushConstantsInfo).dot(Field::layout),
+                             "is VK_NULL_HANDLE and pNext is missing VkPipelineLayoutCreateInfo.");
+        }
+    }
+    return skip;
 }
 
 bool StatelessValidation::manual_PreCallValidateCmdClearColorImage(VkCommandBuffer commandBuffer, VkImage image,
@@ -405,15 +449,18 @@ bool StatelessValidation::manual_PreCallValidateCmdUpdateBuffer(VkCommandBuffer
     bool skip = false;
 
     if (dstOffset & 3) {
-        skip |= LogError("VUID-vkCmdUpdateBuffer-dstOffset-00036", commandBuffer, error_obj.location.dot(Field::dstOffset),
+        const LogObjectList objlist(commandBuffer, dstBuffer);
+        skip |= LogError("VUID-vkCmdUpdateBuffer-dstOffset-00036", objlist, error_obj.location.dot(Field::dstOffset),
                          "(%" PRIu64 "), is not a multiple of 4.", dstOffset);
     }
 
     if ((dataSize <= 0) || (dataSize > 65536)) {
-        skip |= LogError("VUID-vkCmdUpdateBuffer-dataSize-00037", commandBuffer, error_obj.location.dot(Field::dataSize),
+        const LogObjectList objlist(commandBuffer, dstBuffer);
+        skip |= LogError("VUID-vkCmdUpdateBuffer-dataSize-00037", objlist, error_obj.location.dot(Field::dataSize),
                          "(%" PRIu64 "), must be greater than zero and less than or equal to 65536.", dataSize);
     } else if (dataSize & 3) {
-        skip |= LogError("VUID-vkCmdUpdateBuffer-dataSize-00038", commandBuffer, error_obj.location.dot(Field::dataSize),
+        const LogObjectList objlist(commandBuffer, dstBuffer);
+        skip |= LogError("VUID-vkCmdUpdateBuffer-dataSize-00038", objlist, error_obj.location.dot(Field::dataSize),
                          "(%" PRIu64 "), is not a multiple of 4.", dataSize);
     }
     return skip;
@@ -425,22 +472,61 @@ bool StatelessValidation::manual_PreCallValidateCmdFillBuffer(VkCommandBuffer co
     bool skip = false;
 
     if (dstOffset & 3) {
-        skip |= LogError("VUID-vkCmdFillBuffer-dstOffset-00025", dstBuffer, error_obj.location.dot(Field::dstOffset),
+        const LogObjectList objlist(commandBuffer, dstBuffer);
+        skip |= LogError("VUID-vkCmdFillBuffer-dstOffset-00025", objlist, error_obj.location.dot(Field::dstOffset),
                          "(%" PRIu64 ") is not a multiple of 4.", dstOffset);
     }
 
     if (size != VK_WHOLE_SIZE) {
         if (size <= 0) {
-            skip |= LogError("VUID-vkCmdFillBuffer-size-00026", dstBuffer, error_obj.location.dot(Field::size),
+            const LogObjectList objlist(commandBuffer, dstBuffer);
+            skip |= LogError("VUID-vkCmdFillBuffer-size-00026", objlist, error_obj.location.dot(Field::size),
                              "(%" PRIu64 ") must be greater than zero.", size);
         } else if (size & 3) {
-            skip |= LogError("VUID-vkCmdFillBuffer-size-00028", dstBuffer, error_obj.location.dot(Field::size),
+            const LogObjectList objlist(commandBuffer, dstBuffer);
+            skip |= LogError("VUID-vkCmdFillBuffer-size-00028", objlist, error_obj.location.dot(Field::size),
                              "(%" PRIu64 ") is not a multiple of 4.", size);
         }
     }
     return skip;
 }
 
+bool StatelessValidation::manual_PreCallValidateCmdBindDescriptorBuffersEXT(VkCommandBuffer commandBuffer, uint32_t bufferCount,
+                                                                            const VkDescriptorBufferBindingInfoEXT *pBindingInfos,
+                                                                            const ErrorObject &error_obj) const {
+    bool skip = false;
+    if (!enabled_features.descriptorBuffer) {
+        skip |= LogError("VUID-vkCmdBindDescriptorBuffersEXT-None-08047", commandBuffer, error_obj.location,
+                         "descriptorBuffer feature was not enabled.");
+    }
+
+    for (uint32_t i = 0; i < bufferCount; i++) {
+        if (!vku::FindStructInPNextChain<VkBufferUsageFlags2CreateInfoKHR>(pBindingInfos[i].pNext)) {
+            skip |= ValidateFlags(error_obj.location.dot(Field::pBindingInfos, i).dot(Field::usage),
+                                  vvl::FlagBitmask::VkBufferUsageFlagBits, AllVkBufferUsageFlagBits, pBindingInfos[i].usage,
+                                  kRequiredFlags, "VUID-VkDescriptorBufferBindingInfoEXT-None-09499",
+                                  "VUID-VkDescriptorBufferBindingInfoEXT-None-09500");
+        }
+    }
+
+    return skip;
+}
+
+bool StatelessValidation::manual_PreCallValidateGetPhysicalDeviceExternalBufferProperties(
+    VkPhysicalDevice physicalDevice, const VkPhysicalDeviceExternalBufferInfo *pExternalBufferInfo,
+    VkExternalBufferProperties *pExternalBufferProperties, const ErrorObject &error_obj) const {
+    bool skip = false;
+
+    if (!vku::FindStructInPNextChain<VkBufferUsageFlags2CreateInfoKHR>(pExternalBufferInfo->pNext)) {
+        skip |= ValidateFlags(error_obj.location.dot(Field::pExternalBufferInfo).dot(Field::usage),
+                              vvl::FlagBitmask::VkBufferUsageFlagBits, AllVkBufferUsageFlagBits, pExternalBufferInfo->usage,
+                              kRequiredFlags, "VUID-VkPhysicalDeviceExternalBufferInfo-None-09499",
+                              "VUID-VkPhysicalDeviceExternalBufferInfo-None-09500");
+    }
+
+    return skip;
+}
+
 bool StatelessValidation::manual_PreCallValidateCmdPushDescriptorSetKHR(
     VkCommandBuffer commandBuffer, VkPipelineBindPoint pipelineBindPoint, VkPipelineLayout layout, uint32_t set,
     uint32_t descriptorWriteCount, const VkWriteDescriptorSet *pDescriptorWrites, const ErrorObject &error_obj) const {
@@ -450,8 +536,20 @@ bool StatelessValidation::manual_PreCallValidateCmdPushDescriptorSetKHR(
 bool StatelessValidation::manual_PreCallValidateCmdPushDescriptorSet2KHR(VkCommandBuffer commandBuffer,
                                                                          const VkPushDescriptorSetInfoKHR *pPushDescriptorSetInfo,
                                                                          const ErrorObject &error_obj) const {
-    return ValidateWriteDescriptorSet(error_obj.location, pPushDescriptorSetInfo->descriptorWriteCount,
-                                      pPushDescriptorSetInfo->pDescriptorWrites);
+    bool skip = false;
+    skip |= ValidateWriteDescriptorSet(error_obj.location, pPushDescriptorSetInfo->descriptorWriteCount,
+                                       pPushDescriptorSetInfo->pDescriptorWrites);
+    if (pPushDescriptorSetInfo->layout == VK_NULL_HANDLE) {
+        if (!enabled_features.dynamicPipelineLayout) {
+            skip |= LogError("VUID-VkPushDescriptorSetInfoKHR-None-09495", commandBuffer,
+                             error_obj.location.dot(Field::pPushDescriptorSetInfo).dot(Field::layout), "is VK_NULL_HANDLE.");
+        } else if (vku::FindStructInPNextChain<VkPipelineLayoutCreateInfo>(pPushDescriptorSetInfo->pNext)) {
+            skip |= LogError("VUID-VkPushDescriptorSetInfoKHR-layout-09496", commandBuffer,
+                             error_obj.location.dot(Field::pPushDescriptorSetInfo).dot(Field::layout),
+                             "is VK_NULL_HANDLE and pNext is missing VkPipelineLayoutCreateInfo.");
+        }
+    }
+    return skip;
 }
 
 bool StatelessValidation::ValidateViewport(const VkViewport &viewport, VkCommandBuffer object, const Location &loc) const {
@@ -603,13 +701,7 @@ bool StatelessValidation::manual_PreCallValidateBeginCommandBuffer(VkCommandBuff
     bool skip = false;
 
     // VkCommandBufferInheritanceInfo validation, due to a 'noautovalidity' of pBeginInfo->pInheritanceInfo in vkBeginCommandBuffer
-    bool cb_is_secondary;
-    {
-        auto lock = CBReadLock();
-        cb_is_secondary = (secondary_cb_map.find(commandBuffer) != secondary_cb_map.end());
-    }
-
-    if (!cb_is_secondary) {
+    if (!error_obj.handle_data->command_buffer.is_secondary) {
         return skip;
     }
     // Implicit VUs
@@ -619,15 +711,16 @@ bool StatelessValidation::manual_PreCallValidateBeginCommandBuffer(VkCommandBuff
     const VkCommandBufferInheritanceInfo *info = pBeginInfo->pInheritanceInfo;
     const Location begin_info_loc = error_obj.location.dot(Field::pBeginInfo);
     const Location inheritance_loc = begin_info_loc.dot(Field::pInheritanceInfo);
-    skip |= ValidateStructType(inheritance_loc, "VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_INFO", info,
-                               VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_INFO, k_not_required, k_no_vuid,
+    skip |= ValidateStructType(inheritance_loc, info, VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_INFO, k_not_required, k_no_vuid,
                                "VUID-VkCommandBufferInheritanceInfo-sType-sType");
 
     if (info) {
         constexpr std::array allowed_structs = {VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_CONDITIONAL_RENDERING_INFO_EXT,
                                                 VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_RENDERING_INFO_KHR,
                                                 VK_STRUCTURE_TYPE_ATTACHMENT_SAMPLE_COUNT_INFO_AMD,
-                                                VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_VIEWPORT_SCISSOR_INFO_NV};
+                                                VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_VIEWPORT_SCISSOR_INFO_NV,
+                                                VK_STRUCTURE_TYPE_RENDERING_INPUT_ATTACHMENT_INDEX_INFO_KHR,
+                                                VK_STRUCTURE_TYPE_RENDERING_ATTACHMENT_LOCATION_INFO_KHR};
         skip |= ValidateStructPnext(inheritance_loc, info->pNext, allowed_structs.size(), allowed_structs.data(),
                                     GeneratedVulkanHeaderVersion, "VUID-VkCommandBufferInheritanceInfo-pNext-pNext",
                                     "VUID-VkCommandBufferInheritanceInfo-sType-unique");
@@ -635,24 +728,26 @@ bool StatelessValidation::manual_PreCallValidateBeginCommandBuffer(VkCommandBuff
         skip |= ValidateBool32(inheritance_loc.dot(Field::occlusionQueryEnable), info->occlusionQueryEnable);
 
         // Explicit VUs
-        if (!physical_device_features.inheritedQueries && info->occlusionQueryEnable == VK_TRUE) {
+        if (!enabled_features.inheritedQueries && info->occlusionQueryEnable == VK_TRUE) {
             skip |= LogError(
                 "VUID-VkCommandBufferInheritanceInfo-occlusionQueryEnable-00056", commandBuffer, error_obj.location,
                 "Inherited queries feature is disabled, but pBeginInfo->pInheritanceInfo->occlusionQueryEnable is VK_TRUE.");
         }
 
-        if (physical_device_features.inheritedQueries) {
-            skip |= ValidateFlags(inheritance_loc.dot(Field::queryFlags), "VkQueryControlFlagBits", AllVkQueryControlFlagBits,
-                                  info->queryFlags, kOptionalFlags, "VUID-VkCommandBufferInheritanceInfo-queryFlags-00057");
+        if (enabled_features.inheritedQueries) {
+            skip |= ValidateFlags(inheritance_loc.dot(Field::queryFlags), vvl::FlagBitmask::VkQueryControlFlagBits,
+                                  AllVkQueryControlFlagBits, info->queryFlags, kOptionalFlags,
+                                  "VUID-VkCommandBufferInheritanceInfo-queryFlags-00057");
         } else {  // !inheritedQueries
             skip |= ValidateReservedFlags(inheritance_loc.dot(Field::queryFlags), info->queryFlags,
                                           "VUID-VkCommandBufferInheritanceInfo-queryFlags-02788");
         }
 
-        if (physical_device_features.pipelineStatisticsQuery) {
-            skip |= ValidateFlags(inheritance_loc.dot(Field::pipelineStatistics), "VkQueryPipelineStatisticFlagBits",
-                                  AllVkQueryPipelineStatisticFlagBits, info->pipelineStatistics, kOptionalFlags,
-                                  "VUID-VkCommandBufferInheritanceInfo-pipelineStatistics-02789");
+        if (enabled_features.pipelineStatisticsQuery) {
+            skip |=
+                ValidateFlags(inheritance_loc.dot(Field::pipelineStatistics), vvl::FlagBitmask::VkQueryPipelineStatisticFlagBits,
+                              AllVkQueryPipelineStatisticFlagBits, info->pipelineStatistics, kOptionalFlags,
+                              "VUID-VkCommandBufferInheritanceInfo-pipelineStatistics-02789");
         } else {  // !pipelineStatisticsQuery
             skip |= ValidateReservedFlags(inheritance_loc.dot(Field::pipelineStatistics), info->pipelineStatistics,
                                           "VUID-VkCommandBufferInheritanceInfo-pipelineStatistics-00058");
@@ -660,9 +755,7 @@ bool StatelessValidation::manual_PreCallValidateBeginCommandBuffer(VkCommandBuff
 
         const auto *conditional_rendering = vku::FindStructInPNextChain<VkCommandBufferInheritanceConditionalRenderingInfoEXT>(info->pNext);
         if (conditional_rendering) {
-            const auto *cr_features = vku::FindStructInPNextChain<VkPhysicalDeviceConditionalRenderingFeaturesEXT>(device_createinfo_pnext);
-            const auto inherited_conditional_rendering = cr_features && cr_features->inheritedConditionalRendering;
-            if (!inherited_conditional_rendering && conditional_rendering->conditionalRenderingEnable == VK_TRUE) {
+            if (!enabled_features.inheritedConditionalRendering && conditional_rendering->conditionalRenderingEnable == VK_TRUE) {
                 skip |= LogError(
                     "VUID-VkCommandBufferInheritanceConditionalRenderingInfoEXT-conditionalRenderingEnable-01977", commandBuffer,
                     error_obj.location,
@@ -672,7 +765,7 @@ bool StatelessValidation::manual_PreCallValidateBeginCommandBuffer(VkCommandBuff
         }
 
         auto p_inherited_viewport_scissor_info = vku::FindStructInPNextChain<VkCommandBufferInheritanceViewportScissorInfoNV>(info->pNext);
-        if (p_inherited_viewport_scissor_info != nullptr && !physical_device_features.multiViewport &&
+        if (p_inherited_viewport_scissor_info != nullptr && !enabled_features.multiViewport &&
             p_inherited_viewport_scissor_info->viewportScissor2D == VK_TRUE &&
             p_inherited_viewport_scissor_info->viewportDepthCount != 1) {
             skip |= LogError("VUID-VkCommandBufferInheritanceViewportScissorInfoNV-viewportScissor2D-04783", commandBuffer,
@@ -684,34 +777,3 @@ bool StatelessValidation::manual_PreCallValidateBeginCommandBuffer(VkCommandBuff
     }
     return skip;
 }
-
-void StatelessValidation::PostCallRecordAllocateCommandBuffers(VkDevice device, const VkCommandBufferAllocateInfo *pAllocateInfo,
-                                                               VkCommandBuffer *pCommandBuffers, const RecordObject &record_obj) {
-    if ((record_obj.result == VK_SUCCESS) && pAllocateInfo && (pAllocateInfo->level == VK_COMMAND_BUFFER_LEVEL_SECONDARY)) {
-        auto lock = CBWriteLock();
-        for (uint32_t cb_index = 0; cb_index < pAllocateInfo->commandBufferCount; cb_index++) {
-            secondary_cb_map.emplace(pCommandBuffers[cb_index], pAllocateInfo->commandPool);
-        }
-    }
-}
-
-void StatelessValidation::PostCallRecordFreeCommandBuffers(VkDevice device, VkCommandPool commandPool, uint32_t commandBufferCount,
-                                                           const VkCommandBuffer *pCommandBuffers, const RecordObject &record_obj) {
-    auto lock = CBWriteLock();
-    for (uint32_t cb_index = 0; cb_index < commandBufferCount; cb_index++) {
-        secondary_cb_map.erase(pCommandBuffers[cb_index]);
-    }
-}
-
-void StatelessValidation::PostCallRecordDestroyCommandPool(VkDevice device, VkCommandPool commandPool,
-                                                           const VkAllocationCallbacks *pAllocator,
-                                                           const RecordObject &record_obj) {
-    auto lock = CBWriteLock();
-    for (auto item = secondary_cb_map.begin(); item != secondary_cb_map.end();) {
-        if (item->second == commandPool) {
-            item = secondary_cb_map.erase(item);
-        } else {
-            ++item;
-        }
-    }
-}
