@@ -1,7 +1,7 @@
-/* Copyright (c) 2015-2023 The Khronos Group Inc.
- * Copyright (c) 2015-2023 Valve Corporation
- * Copyright (c) 2015-2023 LunarG, Inc.
- * Copyright (C) 2015-2023 Google Inc.
+/* Copyright (c) 2015-2024 The Khronos Group Inc.
+ * Copyright (c) 2015-2024 Valve Corporation
+ * Copyright (c) 2015-2024 LunarG, Inc.
+ * Copyright (C) 2015-2024 Google Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -155,13 +155,13 @@ bool StatelessValidation::manual_PreCallValidateCreateImage(VkDevice device, con
                          string_VkImageType(pCreateInfo->imageType));
     }
 
-    if ((image_flags & VK_IMAGE_CREATE_SPARSE_BINDING_BIT) && (!physical_device_features.sparseBinding)) {
+    if ((image_flags & VK_IMAGE_CREATE_SPARSE_BINDING_BIT) && (!enabled_features.sparseBinding)) {
         skip |= LogError("VUID-VkImageCreateInfo-flags-00969", device, create_info_loc.dot(Field::flags),
                          "includes VK_IMAGE_CREATE_SPARSE_BINDING_BIT, but the "
                          "sparseBinding feature was not enabled.");
     }
 
-    if ((image_flags & VK_IMAGE_CREATE_SPARSE_ALIASED_BIT) && (!physical_device_features.sparseResidencyAliased)) {
+    if ((image_flags & VK_IMAGE_CREATE_SPARSE_ALIASED_BIT) && (!enabled_features.sparseResidencyAliased)) {
         skip |= LogError("VUID-VkImageCreateInfo-flags-01924", device, create_info_loc.dot(Field::flags),
                          "includes VK_IMAGE_CREATE_SPARSE_ALIASED_BIT but the sparseResidencyAliased feature was not enabled.");
     }
@@ -189,14 +189,14 @@ bool StatelessValidation::manual_PreCallValidateCreateImage(VkDevice device, con
         }
 
         // Sparse 2D image when device doesn't support it
-        if ((!physical_device_features.sparseResidencyImage2D) && (VK_IMAGE_TYPE_2D == pCreateInfo->imageType)) {
+        if ((!enabled_features.sparseResidencyImage2D) && (VK_IMAGE_TYPE_2D == pCreateInfo->imageType)) {
             skip |= LogError("VUID-VkImageCreateInfo-imageType-00971", device, create_info_loc.dot(Field::flags),
                              "includes VK_IMAGE_CREATE_SPARSE_BINDING_BIT and imageType is VK_IMAGE_TYPE_2D, but "
                              "sparseResidencyImage2D feature was not enabled.");
         }
 
         // Sparse 3D image when device doesn't support it
-        if ((!physical_device_features.sparseResidencyImage3D) && (VK_IMAGE_TYPE_3D == pCreateInfo->imageType)) {
+        if ((!enabled_features.sparseResidencyImage3D) && (VK_IMAGE_TYPE_3D == pCreateInfo->imageType)) {
             skip |= LogError("VUID-VkImageCreateInfo-imageType-00972", device, create_info_loc.dot(Field::flags),
                              "includes VK_IMAGE_CREATE_SPARSE_BINDING_BIT and imageType is VK_IMAGE_TYPE_3D, but "
                              "sparseResidencyImage3D feature was not enabled.");
@@ -204,19 +204,19 @@ bool StatelessValidation::manual_PreCallValidateCreateImage(VkDevice device, con
 
         // Multi-sample 2D image when device doesn't support it
         if (VK_IMAGE_TYPE_2D == pCreateInfo->imageType) {
-            if ((!physical_device_features.sparseResidency2Samples) && (VK_SAMPLE_COUNT_2_BIT == pCreateInfo->samples)) {
+            if ((!enabled_features.sparseResidency2Samples) && (VK_SAMPLE_COUNT_2_BIT == pCreateInfo->samples)) {
                 skip |= LogError("VUID-VkImageCreateInfo-imageType-00973", device, create_info_loc.dot(Field::flags),
                                  "includes VK_IMAGE_CREATE_SPARSE_BINDING_BIT and imageType is VK_IMAGE_TYPE_2D and samples is "
                                  "VK_SAMPLE_COUNT_2_BIT, but sparseResidency2Samples feature was not enabled.");
-            } else if ((!physical_device_features.sparseResidency4Samples) && (VK_SAMPLE_COUNT_4_BIT == pCreateInfo->samples)) {
+            } else if ((!enabled_features.sparseResidency4Samples) && (VK_SAMPLE_COUNT_4_BIT == pCreateInfo->samples)) {
                 skip |= LogError("VUID-VkImageCreateInfo-imageType-00974", device, create_info_loc.dot(Field::flags),
                                  "includes VK_IMAGE_CREATE_SPARSE_BINDING_BIT and imageType is VK_IMAGE_TYPE_2D and samples is "
                                  "VK_SAMPLE_COUNT_4_BIT, but sparseResidency4Samples feature was not enabled.");
-            } else if ((!physical_device_features.sparseResidency8Samples) && (VK_SAMPLE_COUNT_8_BIT == pCreateInfo->samples)) {
+            } else if ((!enabled_features.sparseResidency8Samples) && (VK_SAMPLE_COUNT_8_BIT == pCreateInfo->samples)) {
                 skip |= LogError("VUID-VkImageCreateInfo-imageType-00975", device, create_info_loc.dot(Field::flags),
                                  "includes VK_IMAGE_CREATE_SPARSE_BINDING_BIT and imageType is VK_IMAGE_TYPE_2D and samples is "
                                  "VK_SAMPLE_COUNT_8_BIT, but sparseResidency8Samples feature was not enabled.");
-            } else if ((!physical_device_features.sparseResidency16Samples) && (VK_SAMPLE_COUNT_16_BIT == pCreateInfo->samples)) {
+            } else if ((!enabled_features.sparseResidency16Samples) && (VK_SAMPLE_COUNT_16_BIT == pCreateInfo->samples)) {
                 skip |= LogError("VUID-VkImageCreateInfo-imageType-00976", device, create_info_loc.dot(Field::flags),
                                  "includes VK_IMAGE_CREATE_SPARSE_BINDING_BIT and imageType is VK_IMAGE_TYPE_2D and samples is "
                                  "VK_SAMPLE_COUNT_16_BIT, but sparseResidency16Samples feature was not enabled.");
@@ -238,10 +238,7 @@ bool StatelessValidation::manual_PreCallValidateCreateImage(VkDevice device, con
                              "alias VK_IMAGE_USAGE_SHADING_RATE_IMAGE_BIT_NV), but samples is %s.",
                              string_VkSampleCountFlagBits(pCreateInfo->samples));
         }
-        const auto *shading_rate_image_features =
-            vku::FindStructInPNextChain<VkPhysicalDeviceShadingRateImageFeaturesNV>(device_createinfo_pnext);
-        if (shading_rate_image_features && shading_rate_image_features->shadingRateImage &&
-            pCreateInfo->tiling != VK_IMAGE_TILING_OPTIMAL) {
+        if (enabled_features.shadingRateImage && pCreateInfo->tiling != VK_IMAGE_TILING_OPTIMAL) {
             // KHR flag can be non-optimal
             skip |= LogError("VUID-VkImageCreateInfo-shadingRateImage-07727", device, create_info_loc.dot(Field::usage),
                              "includes VK_IMAGE_USAGE_SHADING_RATE_IMAGE_BIT_NV, tiling must be "
@@ -321,7 +318,7 @@ bool StatelessValidation::manual_PreCallValidateCreateImage(VkDevice device, con
                 }
             }
 
-            if (!physical_device_features.shaderStorageImageMultisample &&
+            if (!enabled_features.shaderStorageImageMultisample &&
                 ((image_stencil_struct->stencilUsage & VK_IMAGE_USAGE_STORAGE_BIT) != 0) &&
                 (pCreateInfo->samples != VK_SAMPLE_COUNT_1_BIT)) {
                 skip |= LogError("VUID-VkImageCreateInfo-format-02538", device,
@@ -365,7 +362,7 @@ bool StatelessValidation::manual_PreCallValidateCreateImage(VkDevice device, con
         }
     }
 
-    if ((!physical_device_features.shaderStorageImageMultisample) && ((pCreateInfo->usage & VK_IMAGE_USAGE_STORAGE_BIT) != 0) &&
+    if ((!enabled_features.shaderStorageImageMultisample) && ((pCreateInfo->usage & VK_IMAGE_USAGE_STORAGE_BIT) != 0) &&
         (pCreateInfo->samples != VK_SAMPLE_COUNT_1_BIT)) {
         skip |= LogError("VUID-VkImageCreateInfo-usage-00968", device, create_info_loc.dot(Field::usage),
                          "includes VK_IMAGE_USAGE_STORAGE_BIT and imageType is %s, but shaderStorageImageMultisample feature "
@@ -594,24 +591,26 @@ bool StatelessValidation::manual_PreCallValidateCreateImage(VkDevice device, con
         }
         // Check if viewFormatCount is not zero that it is all compatible
         for (uint32_t i = 0; i < viewFormatCount; i++) {
-            const bool class_compatible =
-                vkuFormatCompatibilityClass(format_list_info->pViewFormats[i]) == vkuFormatCompatibilityClass(image_format);
-            if (!class_compatible) {
+            const VkFormat view_format = format_list_info->pViewFormats[i];
+            const Location format_loc = create_info_loc.pNext(Struct::VkImageFormatListCreateInfo, Field::pViewFormats, i);
+            const bool class_compatible = vkuFormatCompatibilityClass(view_format) == vkuFormatCompatibilityClass(image_format);
+
+            if (view_format == VK_FORMAT_UNDEFINED) {
+                skip |= LogError("VUID-VkImageFormatListCreateInfo-viewFormatCount-09540", device, format_loc,
+                                 "is VK_FORMAT_UNDEFINED.");
+            } else if (!class_compatible) {
                 if (image_flags & VK_IMAGE_CREATE_BLOCK_TEXEL_VIEW_COMPATIBLE_BIT) {
-                    const bool size_compatible =
-                        !vkuFormatIsCompressed(format_list_info->pViewFormats[i]) &&
-                        vkuFormatElementSize(format_list_info->pViewFormats[i]) == vkuFormatElementSize(image_format);
+                    const bool size_compatible = !vkuFormatIsCompressed(view_format) &&
+                                                 vkuFormatElementSize(view_format) == vkuFormatElementSize(image_format);
                     if (!size_compatible) {
-                        skip |= LogError("VUID-VkImageCreateInfo-pNext-06722", device,
-                                         create_info_loc.pNext(Struct::VkImageFormatListCreateInfo, Field::pViewFormats, i),
+                        skip |= LogError("VUID-VkImageCreateInfo-pNext-06722", device, format_loc,
                                          "(%s) and VkImageCreateInfo::format (%s) are not compatible or size-compatible.",
-                                         string_VkFormat(format_list_info->pViewFormats[i]), string_VkFormat(image_format));
+                                         string_VkFormat(view_format), string_VkFormat(image_format));
                     }
                 } else {
-                    skip |= LogError("VUID-VkImageCreateInfo-pNext-06722", device,
-                                     create_info_loc.pNext(Struct::VkImageFormatListCreateInfo, Field::pViewFormats, i),
-                                     "(%s) and VkImageCreateInfo::format (%s) are not compatible.",
-                                     string_VkFormat(format_list_info->pViewFormats[i]), string_VkFormat(image_format));
+                    skip |= LogError("VUID-VkImageCreateInfo-pNext-06722", device, format_loc,
+                                     "(%s) and VkImageCreateInfo::format (%s) are not compatible.", string_VkFormat(view_format),
+                                     string_VkFormat(image_format));
                 }
             }
         }
@@ -619,9 +618,10 @@ bool StatelessValidation::manual_PreCallValidateCreateImage(VkDevice device, con
 
     const auto image_compression_control = vku::FindStructInPNextChain<VkImageCompressionControlEXT>(pCreateInfo->pNext);
     if (image_compression_control) {
-        skip |= ValidateFlags(create_info_loc.pNext(Struct::VkImageCompressionControlEXT, Field::flags),
-                              "VkImageCompressionFlagsEXT", AllVkImageCompressionFlagBitsEXT, image_compression_control->flags,
-                              kOptionalSingleBit, "VUID-VkImageCompressionControlEXT-flags-06747");
+        skip |=
+            ValidateFlags(create_info_loc.pNext(Struct::VkImageCompressionControlEXT, Field::flags),
+                          vvl::FlagBitmask::VkImageCompressionFlagBitsEXT, AllVkImageCompressionFlagBitsEXT,
+                          image_compression_control->flags, kOptionalSingleBit, "VUID-VkImageCompressionControlEXT-flags-06747");
 
         if (image_compression_control->flags == VK_IMAGE_COMPRESSION_FIXED_RATE_EXPLICIT_EXT &&
             !image_compression_control->pFixedRateFlags) {
@@ -687,7 +687,7 @@ bool StatelessValidation::manual_PreCallValidateCreateImageView(VkDevice device,
     }
     const Location create_info_loc = error_obj.location.dot(Field::pCreateInfo);
     // Validate feature set if using CUBE_ARRAY
-    if ((pCreateInfo->viewType == VK_IMAGE_VIEW_TYPE_CUBE_ARRAY) && (physical_device_features.imageCubeArray == false)) {
+    if ((pCreateInfo->viewType == VK_IMAGE_VIEW_TYPE_CUBE_ARRAY) && (!enabled_features.imageCubeArray)) {
         skip |= LogError("VUID-VkImageViewCreateInfo-viewType-01004", pCreateInfo->image, create_info_loc.dot(Field::viewType),
                          "is VK_IMAGE_VIEW_TYPE_CUBE_ARRAY but the imageCubeArray feature is not enabled.");
     }
