@@ -1,7 +1,7 @@
-/* Copyright (c) 2015-2023 The Khronos Group Inc.
- * Copyright (c) 2015-2023 Valve Corporation
- * Copyright (c) 2015-2023 LunarG, Inc.
- * Copyright (C) 2015-2023 Google Inc.
+/* Copyright (c) 2015-2024 The Khronos Group Inc.
+ * Copyright (c) 2015-2024 Valve Corporation
+ * Copyright (c) 2015-2024 LunarG, Inc.
+ * Copyright (C) 2015-2024 Google Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -44,86 +44,78 @@ bool StatelessValidation::manual_PreCallValidateAcquireNextImage2KHR(VkDevice de
     return skip;
 }
 
-bool StatelessValidation::ValidateSwapchainCreateInfo(VkSwapchainCreateInfoKHR const *pCreateInfo, const Location &loc) const {
+bool StatelessValidation::ValidateSwapchainCreateInfo(const VkSwapchainCreateInfoKHR &create_info, const Location &loc) const {
     bool skip = false;
 
-    if (pCreateInfo != nullptr) {
-        // Validation for parameters excluded from the generated validation code due to a 'noautovalidity' tag in vk.xml
-        if (pCreateInfo->imageSharingMode == VK_SHARING_MODE_CONCURRENT) {
-            // If imageSharingMode is VK_SHARING_MODE_CONCURRENT, queueFamilyIndexCount must be greater than 1
-            if (pCreateInfo->queueFamilyIndexCount <= 1) {
-                skip |= LogError("VUID-VkSwapchainCreateInfoKHR-imageSharingMode-01278", device, loc.dot(Field::imageSharingMode),
-                                 "is VK_SHARING_MODE_CONCURRENT, but queueFamilyIndexCount is %" PRIu32 ".",
-                                 pCreateInfo->queueFamilyIndexCount);
-            }
-
-            // If imageSharingMode is VK_SHARING_MODE_CONCURRENT, pQueueFamilyIndices must be a pointer to an array of
-            // queueFamilyIndexCount uint32_t values
-            if (pCreateInfo->pQueueFamilyIndices == nullptr) {
-                skip |= LogError("VUID-VkSwapchainCreateInfoKHR-imageSharingMode-01277", device, loc.dot(Field::imageSharingMode),
-                                 "is VK_SHARING_MODE_CONCURRENT, but pQueueFamilyIndices is NULL.");
-            }
+    // Validation for parameters excluded from the generated validation code due to a 'noautovalidity' tag in vk.xml
+    if (create_info.imageSharingMode == VK_SHARING_MODE_CONCURRENT) {
+        // If imageSharingMode is VK_SHARING_MODE_CONCURRENT, queueFamilyIndexCount must be greater than 1
+        if (create_info.queueFamilyIndexCount <= 1) {
+            skip |= LogError("VUID-VkSwapchainCreateInfoKHR-imageSharingMode-01278", device, loc.dot(Field::imageSharingMode),
+                             "is VK_SHARING_MODE_CONCURRENT, but queueFamilyIndexCount is %" PRIu32 ".",
+                             create_info.queueFamilyIndexCount);
         }
 
-        skip |= ValidateNotZero(pCreateInfo->imageArrayLayers == 0, "VUID-VkSwapchainCreateInfoKHR-imageArrayLayers-01275",
-                                loc.dot(Field::imageArrayLayers));
+        // If imageSharingMode is VK_SHARING_MODE_CONCURRENT, pQueueFamilyIndices must be a pointer to an array of
+        // queueFamilyIndexCount uint32_t values
+        if (create_info.pQueueFamilyIndices == nullptr) {
+            skip |= LogError("VUID-VkSwapchainCreateInfoKHR-imageSharingMode-01277", device, loc.dot(Field::imageSharingMode),
+                             "is VK_SHARING_MODE_CONCURRENT, but pQueueFamilyIndices is NULL.");
+        }
+    }
 
-        // Validate VK_KHR_image_format_list VkImageFormatListCreateInfo
-        const auto format_list_info = vku::FindStructInPNextChain<VkImageFormatListCreateInfo>(pCreateInfo->pNext);
-        if (format_list_info) {
-            const uint32_t viewFormatCount = format_list_info->viewFormatCount;
-            if (((pCreateInfo->flags & VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR) == 0) && (viewFormatCount > 1)) {
-                skip |= LogError("VUID-VkSwapchainCreateInfoKHR-flags-04100", device,
-                                 loc.pNext(Struct::VkImageFormatListCreateInfo, Field::viewFormatCount),
-                                 "is %" PRIu32 " but flag (%s) does not includes VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR.",
-                                 viewFormatCount, string_VkImageCreateFlags(pCreateInfo->flags).c_str());
-            }
+    skip |= ValidateNotZero(create_info.imageArrayLayers == 0, "VUID-VkSwapchainCreateInfoKHR-imageArrayLayers-01275",
+                            loc.dot(Field::imageArrayLayers));
 
-            // Using the first format, compare the rest of the formats against it that they are compatible
-            for (uint32_t i = 1; i < viewFormatCount; i++) {
-                if (vkuFormatCompatibilityClass(format_list_info->pViewFormats[0]) !=
-                    vkuFormatCompatibilityClass(format_list_info->pViewFormats[i])) {
-                    skip |= LogError("VUID-VkSwapchainCreateInfoKHR-pNext-04099", device,
-                                     loc.pNext(Struct::VkImageFormatListCreateInfo, Field::pViewFormats, i),
-                                     "(%s) and pViewFormats[0] (%s) are not compatible in the pNext chain.",
-                                     string_VkFormat(format_list_info->pViewFormats[i]),
-                                     string_VkFormat(format_list_info->pViewFormats[0]));
-                }
-            }
+    // Validate VK_KHR_image_format_list VkImageFormatListCreateInfo
+    const auto format_list_info = vku::FindStructInPNextChain<VkImageFormatListCreateInfo>(create_info.pNext);
+    if (format_list_info) {
+        const uint32_t viewFormatCount = format_list_info->viewFormatCount;
+        if (((create_info.flags & VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR) == 0) && (viewFormatCount > 1)) {
+            skip |= LogError("VUID-VkSwapchainCreateInfoKHR-flags-04100", device,
+                             loc.pNext(Struct::VkImageFormatListCreateInfo, Field::viewFormatCount),
+                             "is %" PRIu32 " but flag (%s) does not includes VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR.",
+                             viewFormatCount, string_VkImageCreateFlags(create_info.flags).c_str());
         }
 
-        // Validate VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR
-        if ((pCreateInfo->flags & VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR) != 0) {
-            if (!IsExtEnabled(device_extensions.vk_khr_swapchain_mutable_format)) {
-                skip |= LogError(kVUID_PVError_ExtensionNotEnabled, device, loc.dot(Field::flags),
-                                 "includes VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR which requires the "
-                                 "VK_KHR_swapchain_mutable_format extension, which has not been enabled.");
-            } else {
-                if (format_list_info == nullptr) {
-                    skip |= LogError("VUID-VkSwapchainCreateInfoKHR-flags-03168", device, loc.dot(Field::flags),
-                                     "includes VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR but the pNext does not contain "
-                                     "VkImageFormatListCreateInfo.");
-                } else if (format_list_info->viewFormatCount == 0) {
-                    skip |= LogError("VUID-VkSwapchainCreateInfoKHR-flags-03168", device, loc.dot(Field::flags),
-                                     "includes VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR but %s is zero.",
-                                     loc.pNext(Struct::VkImageFormatListCreateInfo, Field::viewFormatCount).Fields().c_str());
-                } else {
-                    bool found_base_format = false;
-                    for (uint32_t i = 0; i < format_list_info->viewFormatCount; ++i) {
-                        if (format_list_info->pViewFormats[i] == pCreateInfo->imageFormat) {
-                            found_base_format = true;
-                            break;
-                        }
-                    }
-                    if (!found_base_format) {
-                        skip |= LogError("VUID-VkSwapchainCreateInfoKHR-flags-03168", device, loc.dot(Field::flags),
-                                         "includes VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR but none of the "
-                                         "elements of the pViewFormats member of VkImageFormatListCreateInfo match "
-                                         "imageFormat (%s).",
-                                         string_VkFormat(pCreateInfo->imageFormat));
-                    }
+        // Using the first format, compare the rest of the formats against it that they are compatible
+        for (uint32_t i = 1; i < viewFormatCount; i++) {
+            if (vkuFormatCompatibilityClass(format_list_info->pViewFormats[0]) !=
+                vkuFormatCompatibilityClass(format_list_info->pViewFormats[i])) {
+                skip |= LogError("VUID-VkSwapchainCreateInfoKHR-pNext-04099", device,
+                                 loc.pNext(Struct::VkImageFormatListCreateInfo, Field::pViewFormats, i),
+                                 "(%s) and pViewFormats[0] (%s) are not compatible in the pNext chain.",
+                                 string_VkFormat(format_list_info->pViewFormats[i]),
+                                 string_VkFormat(format_list_info->pViewFormats[0]));
+            }
+        }
+    }
+
+    // Validate VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR
+    if ((create_info.flags & VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR) != 0) {
+        if (format_list_info == nullptr) {
+            skip |= LogError("VUID-VkSwapchainCreateInfoKHR-flags-03168", device, loc.dot(Field::flags),
+                             "includes VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR but the pNext does not contain "
+                             "VkImageFormatListCreateInfo.");
+        } else if (format_list_info->viewFormatCount == 0) {
+            skip |= LogError("VUID-VkSwapchainCreateInfoKHR-flags-03168", device, loc.dot(Field::flags),
+                             "includes VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR but %s is zero.",
+                             loc.pNext(Struct::VkImageFormatListCreateInfo, Field::viewFormatCount).Fields().c_str());
+        } else {
+            bool found_base_format = false;
+            for (uint32_t i = 0; i < format_list_info->viewFormatCount; ++i) {
+                if (format_list_info->pViewFormats[i] == create_info.imageFormat) {
+                    found_base_format = true;
+                    break;
                 }
             }
+            if (!found_base_format) {
+                skip |= LogError("VUID-VkSwapchainCreateInfoKHR-flags-03168", device, loc.dot(Field::flags),
+                                 "includes VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR but none of the "
+                                 "elements of the pViewFormats member of VkImageFormatListCreateInfo match "
+                                 "imageFormat (%s).",
+                                 string_VkFormat(create_info.imageFormat));
+            }
         }
     }
     return skip;
@@ -133,7 +125,7 @@ bool StatelessValidation::manual_PreCallValidateCreateSwapchainKHR(VkDevice devi
                                                                    const VkAllocationCallbacks *pAllocator,
                                                                    VkSwapchainKHR *pSwapchain, const ErrorObject &error_obj) const {
     bool skip = false;
-    skip |= ValidateSwapchainCreateInfo(pCreateInfo, error_obj.location.dot(Field::pCreateInfo));
+    skip |= ValidateSwapchainCreateInfo(*pCreateInfo, error_obj.location.dot(Field::pCreateInfo));
     return skip;
 }
 
@@ -145,7 +137,7 @@ bool StatelessValidation::manual_PreCallValidateCreateSharedSwapchainsKHR(VkDevi
     bool skip = false;
     if (pCreateInfos) {
         for (uint32_t i = 0; i < swapchainCount; i++) {
-            skip |= ValidateSwapchainCreateInfo(&pCreateInfos[i], error_obj.location.dot(Field::pCreateInfos, i));
+            skip |= ValidateSwapchainCreateInfo(pCreateInfos[i], error_obj.location.dot(Field::pCreateInfos, i));
         }
     }
     return skip;
@@ -158,12 +150,6 @@ bool StatelessValidation::manual_PreCallValidateQueuePresentKHR(VkQueue queue, c
     if (pPresentInfo && pPresentInfo->pNext) {
         const auto *present_regions = vku::FindStructInPNextChain<VkPresentRegionsKHR>(pPresentInfo->pNext);
         if (present_regions) {
-            // TODO: This and all other pNext extension dependencies should be added to code-generation
-            if (!IsExtEnabled(device_extensions.vk_khr_incremental_present)) {
-                skip |= LogError(kVUID_PVError_ExtensionNotEnabled, device, error_obj.location, "%s extension was not enabled.",
-                                 VK_KHR_INCREMENTAL_PRESENT_EXTENSION_NAME);
-            }
-
             if (present_regions->swapchainCount != pPresentInfo->swapchainCount) {
                 skip |= LogError("VUID-VkPresentRegionsKHR-swapchainCount-01260", device,
                                  error_obj.location.pNext(Struct::VkPresentRegionsKHR, Field::swapchainCount),
@@ -218,7 +204,7 @@ bool StatelessValidation::manual_PreCallValidateGetPhysicalDeviceSurfaceFormatsK
                                                                                    VkSurfaceFormatKHR *pSurfaceFormats,
                                                                                    const ErrorObject &error_obj) const {
     bool skip = false;
-    if (surface == VK_NULL_HANDLE && !instance_extensions.vk_google_surfaceless_query) {
+    if (surface == VK_NULL_HANDLE && !IsExtEnabled(instance_extensions.vk_google_surfaceless_query)) {
         skip |=
             LogError("VUID-vkGetPhysicalDeviceSurfaceFormatsKHR-surface-06524", physicalDevice,
                      error_obj.location.dot(Field::surface), "is VK_NULL_HANDLE and VK_GOOGLE_surfaceless_query is not enabled.");
@@ -232,7 +218,7 @@ bool StatelessValidation::manual_PreCallValidateGetPhysicalDeviceSurfacePresentM
                                                                                         VkPresentModeKHR *pPresentModes,
                                                                                         const ErrorObject &error_obj) const {
     bool skip = false;
-    if (surface == VK_NULL_HANDLE && !instance_extensions.vk_google_surfaceless_query) {
+    if (surface == VK_NULL_HANDLE && !IsExtEnabled(instance_extensions.vk_google_surfaceless_query)) {
         skip |=
             LogError("VUID-vkGetPhysicalDeviceSurfacePresentModesKHR-surface-06524", physicalDevice,
                      error_obj.location.dot(Field::surface), "is VK_NULL_HANDLE and VK_GOOGLE_surfaceless_query is not enabled.");
@@ -244,7 +230,7 @@ bool StatelessValidation::manual_PreCallValidateGetPhysicalDeviceSurfaceCapabili
     VkPhysicalDevice physicalDevice, const VkPhysicalDeviceSurfaceInfo2KHR *pSurfaceInfo,
     VkSurfaceCapabilities2KHR *pSurfaceCapabilities, const ErrorObject &error_obj) const {
     bool skip = false;
-    if (pSurfaceInfo && pSurfaceInfo->surface == VK_NULL_HANDLE && !instance_extensions.vk_google_surfaceless_query) {
+    if (pSurfaceInfo && pSurfaceInfo->surface == VK_NULL_HANDLE && !IsExtEnabled(instance_extensions.vk_google_surfaceless_query)) {
         skip |= LogError("VUID-vkGetPhysicalDeviceSurfaceCapabilities2KHR-pSurfaceInfo-06521", physicalDevice,
                          error_obj.location.dot(Field::pSurfaceInfo).dot(Field::surface),
                          "is VK_NULL_HANDLE and VK_GOOGLE_surfaceless_query is not enabled.");
@@ -303,7 +289,7 @@ bool StatelessValidation::manual_PreCallValidateGetPhysicalDeviceSurfaceFormats2
     VkPhysicalDevice physicalDevice, const VkPhysicalDeviceSurfaceInfo2KHR *pSurfaceInfo, uint32_t *pSurfaceFormatCount,
     VkSurfaceFormat2KHR *pSurfaceFormats, const ErrorObject &error_obj) const {
     bool skip = false;
-    if (pSurfaceInfo && pSurfaceInfo->surface == VK_NULL_HANDLE && !instance_extensions.vk_google_surfaceless_query) {
+    if (pSurfaceInfo && pSurfaceInfo->surface == VK_NULL_HANDLE && !IsExtEnabled(instance_extensions.vk_google_surfaceless_query)) {
         skip |= LogError("VUID-vkGetPhysicalDeviceSurfaceFormats2KHR-pSurfaceInfo-06521", physicalDevice,
                          error_obj.location.dot(Field::pSurfaceInfo).dot(Field::surface),
                          "is VK_NULL_HANDLE and VK_GOOGLE_surfaceless_query is not enabled.");
@@ -316,7 +302,7 @@ bool StatelessValidation::manual_PreCallValidateGetPhysicalDeviceSurfacePresentM
     VkPhysicalDevice physicalDevice, const VkPhysicalDeviceSurfaceInfo2KHR *pSurfaceInfo, uint32_t *pPresentModeCount,
     VkPresentModeKHR *pPresentModes, const ErrorObject &error_obj) const {
     bool skip = false;
-    if (pSurfaceInfo && pSurfaceInfo->surface == VK_NULL_HANDLE && !instance_extensions.vk_google_surfaceless_query) {
+    if (pSurfaceInfo && pSurfaceInfo->surface == VK_NULL_HANDLE && !IsExtEnabled(instance_extensions.vk_google_surfaceless_query)) {
         skip |= LogError("VUID-vkGetPhysicalDeviceSurfacePresentModes2EXT-pSurfaceInfo-06521", physicalDevice,
                          error_obj.location.dot(Field::pSurfaceInfo).dot(Field::surface),
                          "is VK_NULL_HANDLE and VK_GOOGLE_surfaceless_query is not enabled.");
@@ -343,19 +329,23 @@ bool StatelessValidation::PreCallValidateGetDeviceGroupSurfacePresentModes2EXT(V
                                                                                const ErrorObject &error_obj) const {
     bool skip = false;
     if (!IsExtEnabled(device_extensions.vk_khr_swapchain))
-        skip |= OutputExtensionError(error_obj.location, VK_KHR_SWAPCHAIN_EXTENSION_NAME);
+        skip |= OutputExtensionError(error_obj.location, {vvl::Extension::_VK_KHR_swapchain});
     if (!IsExtEnabled(device_extensions.vk_khr_get_surface_capabilities2))
-        skip |= OutputExtensionError(error_obj.location, VK_KHR_GET_SURFACE_CAPABILITIES_2_EXTENSION_NAME);
+        skip |= OutputExtensionError(error_obj.location, {vvl::Extension::_VK_KHR_get_surface_capabilities2});
     if (!IsExtEnabled(device_extensions.vk_khr_surface))
-        skip |= OutputExtensionError(error_obj.location, VK_KHR_SURFACE_EXTENSION_NAME);
+        skip |= OutputExtensionError(error_obj.location, {vvl::Extension::_VK_KHR_surface});
     if (!IsExtEnabled(device_extensions.vk_khr_get_physical_device_properties2))
-        skip |= OutputExtensionError(error_obj.location, VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
+        skip |= OutputExtensionError(error_obj.location, {vvl::Extension::_VK_KHR_get_physical_device_properties2});
     if (!IsExtEnabled(device_extensions.vk_ext_full_screen_exclusive))
-        skip |= OutputExtensionError(error_obj.location, VK_EXT_FULL_SCREEN_EXCLUSIVE_EXTENSION_NAME);
-    skip |= ValidateStructType(error_obj.location.dot(Field::pSurfaceInfo), "VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SURFACE_INFO_2_KHR",
-                               pSurfaceInfo, VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SURFACE_INFO_2_KHR, true,
-                               "VUID-vkGetDeviceGroupSurfacePresentModes2EXT-pSurfaceInfo-parameter",
-                               "VUID-VkPhysicalDeviceSurfaceInfo2KHR-sType-sType");
+        skip |= OutputExtensionError(error_obj.location, {vvl::Extension::_VK_EXT_full_screen_exclusive});
+    if (!pModes) {
+        skip |= LogError("VUID-vkGetDeviceGroupSurfacePresentModes2EXT-pModes-parameter", device,
+                         error_obj.location.dot(Field::pModes), "is NULL.");
+    }
+
+    skip |= ValidateStructType(
+        error_obj.location.dot(Field::pSurfaceInfo), pSurfaceInfo, VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SURFACE_INFO_2_KHR, true,
+        "VUID-vkGetDeviceGroupSurfacePresentModes2EXT-pSurfaceInfo-parameter", "VUID-VkPhysicalDeviceSurfaceInfo2KHR-sType-sType");
     if (pSurfaceInfo != NULL) {
         constexpr std::array allowed_structs = {VK_STRUCTURE_TYPE_SURFACE_FULL_SCREEN_EXCLUSIVE_INFO_EXT,
                                                 VK_STRUCTURE_TYPE_SURFACE_FULL_SCREEN_EXCLUSIVE_WIN32_INFO_EXT};
@@ -365,7 +355,7 @@ bool StatelessValidation::PreCallValidateGetDeviceGroupSurfacePresentModes2EXT(V
                                     "VUID-VkPhysicalDeviceSurfaceInfo2KHR-pNext-pNext",
                                     "VUID-VkPhysicalDeviceSurfaceInfo2KHR-sType-unique");
 
-        if (pSurfaceInfo->surface == VK_NULL_HANDLE && !instance_extensions.vk_google_surfaceless_query) {
+        if (pSurfaceInfo->surface == VK_NULL_HANDLE && !IsExtEnabled(instance_extensions.vk_google_surfaceless_query)) {
             skip |= LogError("VUID-vkGetPhysicalDeviceSurfacePresentModes2EXT-pSurfaceInfo-06521", device,
                              error_obj.location.dot(Field::pSurfaceInfo).dot(Field::surface),
                              "is VK_NULL_HANDLE and VK_GOOGLE_surfaceless_query is not enabled.");
@@ -445,3 +435,18 @@ bool StatelessValidation::manual_PreCallValidateCreateXlibSurfaceKHR(VkInstance
     return skip;
 }
 #endif  // VK_USE_PLATFORM_XLIB_KHR
+
+#ifdef VK_USE_PLATFORM_ANDROID_KHR
+bool StatelessValidation::manual_PreCallValidateCreateAndroidSurfaceKHR(VkInstance instance,
+                                                                        const VkAndroidSurfaceCreateInfoKHR *pCreateInfo,
+                                                                        const VkAllocationCallbacks *pAllocator,
+                                                                        VkSurfaceKHR *pSurface,
+                                                                        const ErrorObject &error_obj) const {
+    bool skip = false;
+    if (pCreateInfo->window == nullptr) {
+        skip |= LogError("VUID-VkAndroidSurfaceCreateInfoKHR-window-01248", instance,
+                         error_obj.location.dot(Field::pCreateInfo).dot(Field::window), "is NULL.");
+    }
+    return skip;
+}
+#endif  // VK_USE_PLATFORM_ANDROID_KHR
