@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2019-2023 Valve Corporation
- * Copyright (c) 2019-2023 LunarG, Inc.
+ * Copyright (c) 2019-2024 Valve Corporation
+ * Copyright (c) 2019-2024 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,14 +17,17 @@
 
 #pragma once
 
-#include "state_tracker/render_pass_state.h"
 #include "sync/sync_common.h"
 #include "sync/sync_access_state.h"
 
+struct SubpassDependencyGraphNode;
+
 namespace vvl {
 class Buffer;
 class VideoSession;
 class VideoPictureResource;
+class Bindable;
+class Event;
 }  // namespace vvl
 
 namespace syncval_state {
@@ -306,7 +309,7 @@ class AccessContext {
                             const ResourceAccessState *infill_state = nullptr, bool recur_to_infill = false);
 
     void UpdateAccessState(const vvl::Buffer &buffer, SyncStageAccessIndex current_usage, SyncOrdering ordering_rule,
-                           const ResourceAccessRange &range, ResourceUsageTag tag);
+                           const ResourceAccessRange &range, ResourceUsageTagEx tag_ex);
     void UpdateAccessState(const ImageState &image, SyncStageAccessIndex current_usage, SyncOrdering ordering_rule,
                            const VkImageSubresourceRange &subresource_range, const ResourceUsageTag &tag);
     void UpdateAccessState(const ImageState &image, SyncStageAccessIndex current_usage, SyncOrdering ordering_rule,
@@ -364,20 +367,22 @@ class AccessContext {
     void EraseIf(Predicate &&pred);
 
     // For use during queue submit building up the QueueBatchContext AccessContext for validation, otherwise clear.
-    void AddAsyncContext(const AccessContext *context, ResourceUsageTag tag);
+    void AddAsyncContext(const AccessContext *context, ResourceUsageTag tag, QueueId queue_id);
 
     class AsyncReference {
       public:
-        AsyncReference(const AccessContext &async_context, ResourceUsageTag async_tag)
-            : context_(&async_context), tag_(async_tag) {}
+        AsyncReference(const AccessContext &async_context, ResourceUsageTag async_tag, QueueId queue_id)
+            : context_(&async_context), tag_(async_tag), queue_id_(queue_id) {}
         const AccessContext &Context() const { return *context_; }
         // For RenderPass time validation this is "start tag", for QueueSubmit, this is the earliest
         // unsynchronized tag for the Queue being tested against (max synchrononous + 1, perhaps)
         ResourceUsageTag StartTag() const;
+        QueueId GetQueueId() const { return queue_id_; }
 
       protected:
         const AccessContext *context_;
         ResourceUsageTag tag_;  // Start of open ended asynchronous range
+        QueueId queue_id_;
     };
 
     template <typename Action, typename RangeGen>
@@ -392,12 +397,12 @@ class AccessContext {
         Iterator Infill(ResourceAccessRangeMap *accesses, const Iterator &pos, const ResourceAccessRange &range) const;
         void operator()(const Iterator &pos) const;
         UpdateMemoryAccessStateFunctor(const AccessContext &context_, SyncStageAccessIndex usage_, SyncOrdering ordering_rule_,
-                                       ResourceUsageTag tag_)
-            : context(context_), usage_info(SyncStageAccess::UsageInfo(usage_)), ordering_rule(ordering_rule_), tag(tag_) {}
+                                       ResourceUsageTagEx tag_ex)
+            : context(context_), usage_info(SyncStageAccess::UsageInfo(usage_)), ordering_rule(ordering_rule_), tag_ex(tag_ex) {}
         const AccessContext &context;
         const SyncStageAccessInfoType &usage_info;
         const SyncOrdering ordering_rule;
-        const ResourceUsageTag tag;
+        const ResourceUsageTagEx tag_ex;
     };
 
     // Follow the context previous to access the access state, supporting "lazy" import into the context. Not intended for
@@ -440,7 +445,8 @@ class AccessContext {
                                       const ResourceAccessRange &range) const;
 
     template <typename Detector, typename RangeGen>
-    HazardResult DetectAsyncHazard(const Detector &detector, const RangeGen &const_range_gen, ResourceUsageTag async_tag) const;
+    HazardResult DetectAsyncHazard(const Detector &detector, const RangeGen &const_range_gen, ResourceUsageTag async_tag,
+                                   QueueId async_queue_id) const;
 
     template <typename NormalizeOp>
     void Trim(NormalizeOp &&normalize);
@@ -513,18 +519,18 @@ void AccessContext::ApplyUpdateAction(const AttachmentViewGen &view_gen, Attachm
 
 // A non recursive range walker for the asynchronous contexts (those we have no barriers with)
 template <typename Detector, typename RangeGen>
-HazardResult AccessContext::DetectAsyncHazard(const Detector &detector, const RangeGen &const_range_gen,
-                                              ResourceUsageTag async_tag) const {
+HazardResult AccessContext::DetectAsyncHazard(const Detector &detector, const RangeGen &const_range_gen, ResourceUsageTag async_tag,
+                                              QueueId async_queue_id) const {
     using RangeType = typename RangeGen::RangeType;
     using ConstIterator = ResourceAccessRangeMap::const_iterator;
     RangeGen range_gen(const_range_gen);
 
     HazardResult hazard;
 
-    auto do_async_hazard_check = [&detector, async_tag, &hazard](const RangeType &range, const ConstIterator &end,
-                                                                 ConstIterator &pos) {
+    auto do_async_hazard_check = [&detector, async_tag, async_queue_id, &hazard](const RangeType &range, const ConstIterator &end,
+                                                                                 ConstIterator &pos) {
         while (pos != end && pos->first.begin < range.end) {
-            hazard = detector.DetectAsync(pos, async_tag);
+            hazard = detector.DetectAsync(pos, async_tag, async_queue_id);
             if (hazard.IsHazard()) return true;
             ++pos;
         }
@@ -655,7 +661,7 @@ HazardResult AccessContext::DetectHazardGeneratedRanges(Detector &detector, Rang
         // Async checks don't require recursive lookups, as the async lists are exhaustive for the top-level context
         // so we'll check these first
         for (const auto &async_ref : async_) {
-            hazard = async_ref.Context().DetectAsyncHazard(detector, range_gen, async_ref.StartTag());
+            hazard = async_ref.Context().DetectAsyncHazard(detector, range_gen, async_ref.StartTag(), async_ref.GetQueueId());
             if (hazard.IsHazard()) return hazard;
         }
     }
