@@ -1,9 +1,9 @@
 #!/usr/bin/env python3
-# Copyright (c) 2021-2023 The Khronos Group Inc.
-# Copyright (c) 2021-2023 Valve Corporation
-# Copyright (c) 2021-2023 LunarG, Inc.
-# Copyright (c) 2021-2023 Google Inc.
-# Copyright (c) 2023-2023 RasterGrid Kft.
+# Copyright (c) 2021-2024 The Khronos Group Inc.
+# Copyright (c) 2021-2024 Valve Corporation
+# Copyright (c) 2021-2024 LunarG, Inc.
+# Copyright (c) 2021-2024 Google Inc.
+# Copyright (c) 2023-2024 RasterGrid Kft.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -27,10 +27,11 @@ import tempfile
 import difflib
 import json
 import common_ci
+import pickle
 from xml.etree import ElementTree
 from generate_spec_error_message import GenerateSpecErrorMessage
 
-def RunGenerators(api: str, registry: str, grammar: str, directory: str, styleFile: str, targetFilter: str):
+def RunGenerators(api: str, registry: str, grammar: str, directory: str, styleFile: str, targetFilter: str, caching: bool):
 
     has_clang_format = shutil.which('clang-format') is not None
     if not has_clang_format:
@@ -63,18 +64,19 @@ def RunGenerators(api: str, registry: str, grammar: str, directory: str, styleFi
     from generators.dynamic_state_generator import DynamicStateOutputGenerator
     from generators.sync_validation_generator import SyncValidationOutputGenerator
     from generators.object_types_generator import ObjectTypesOutputGenerator
-    from generators.safe_struct_generator import SafeStructOutputGenerator
     from generators.enum_flag_bits_generator import EnumFlagBitsOutputGenerator
     from generators.valid_enum_values_generator import ValidEnumValuesOutputGenerator
+    from generators.valid_flag_values_generator import ValidFlagValuesOutputGenerator
     from generators.spirv_tool_commit_id_generator import SpirvToolCommitIdOutputGenerator
     from generators.error_location_helper_generator import ErrorLocationHelperOutputGenerator
     from generators.pnext_chain_extraction_generator import PnextChainExtractionGenerator
-    from generators.state_tracker_helper_generator import StateTrackerHelperOutputGenerator
+    from generators.device_features_generator import DeviceFeaturesOutputGenerator
     from generators.feature_requirements import FeatureRequirementsGenerator
+    from generators.test_icd_generator import TestIcdGenerator
 
     # These set fields that are needed by both OutputGenerator and BaseGenerator,
     # but are uniform and don't need to be set at a per-generated file level
-    from generators.base_generator import SetOutputDirectory, SetTargetApiName, SetMergedApiNames
+    from generators.base_generator import SetOutputDirectory, SetTargetApiName, SetMergedApiNames, EnableCaching
     SetOutputDirectory(directory)
     SetTargetApiName(api)
 
@@ -112,6 +114,7 @@ def RunGenerators(api: str, registry: str, grammar: str, directory: str, styleFi
             'generator' : StatelessValidationHelperOutputGenerator,
             'genCombined': False,
             'options' : [valid_usage_file],
+            'regenerate' : True
         },
         'enum_flag_bits.h' : {
             'generator' : EnumFlagBitsOutputGenerator,
@@ -119,11 +122,15 @@ def RunGenerators(api: str, registry: str, grammar: str, directory: str, styleFi
         },
         'valid_enum_values.h' : {
             'generator' : ValidEnumValuesOutputGenerator,
-            'genCombined': False,
+            'genCombined': True,
         },
         'valid_enum_values.cpp' : {
             'generator' : ValidEnumValuesOutputGenerator,
-            'genCombined': False,
+            'genCombined': True,
+        },
+        'valid_flag_values.cpp' : {
+            'generator' : ValidFlagValuesOutputGenerator,
+            'genCombined': True,
         },
         'object_tracker.h' : {
             'generator' : ObjectTrackerOutputGenerator,
@@ -147,6 +154,10 @@ def RunGenerators(api: str, registry: str, grammar: str, directory: str, styleFi
             'generator' : DispatchTableHelperOutputGenerator,
             'genCombined': True,
         },
+        'vk_dispatch_table_helper.cpp' : {
+            'generator' : DispatchTableHelperOutputGenerator,
+            'genCombined': True,
+        },
         'vk_function_pointers.h' : {
             'generator' : FunctionPointersOutputGenerator,
             'genCombined': True,
@@ -159,43 +170,26 @@ def RunGenerators(api: str, registry: str, grammar: str, directory: str, styleFi
             'generator' : LayerDispatchTableOutputGenerator,
             'genCombined': True,
         },
-        'vk_safe_struct.h' : {
-            'generator' : SafeStructOutputGenerator,
-            'genCombined': True,
-        },
-        'vk_safe_struct_utils.cpp' : {
-            'generator' : SafeStructOutputGenerator,
-            'genCombined': True,
-        },
-        'vk_safe_struct_core.cpp' : {
-            'generator' : SafeStructOutputGenerator,
-            'genCombined': True,
-        },
-        'vk_safe_struct_khr.cpp' : {
-            'generator' : SafeStructOutputGenerator,
-            'genCombined': True,
-        },
-        'vk_safe_struct_ext.cpp' : {
-            'generator' : SafeStructOutputGenerator,
-            'genCombined': True,
-        },
-        'vk_safe_struct_vendor.cpp' : {
-            'generator' : SafeStructOutputGenerator,
-            'genCombined': True,
-        },
         'vk_object_types.h' : {
             'generator' : ObjectTypesOutputGenerator,
             'genCombined': True,
         },
+        'vk_object_types.cpp' : {
+            'generator' : ObjectTypesOutputGenerator,
+            'genCombined': True,
+        },
         'vk_extension_helper.h' : {
             'generator' : ExtensionHelperOutputGenerator,
             'genCombined': True,
         },
+        'vk_extension_helper.cpp' : {
+            'generator' : ExtensionHelperOutputGenerator,
+            'genCombined': True,
+        },
         'vk_api_version.h' : {
             'generator' : ApiVersionOutputGenerator,
             'genCombined': True,
         },
-
         'chassis.h' : {
             'generator' : LayerChassisOutputGenerator,
             'genCombined': True,
@@ -226,15 +220,17 @@ def RunGenerators(api: str, registry: str, grammar: str, directory: str, styleFi
         },
         'sync_validation_types.h' : {
             'generator' : SyncValidationOutputGenerator,
-            'genCombined': False,
+            'genCombined': True,
         },
         'sync_validation_types.cpp' : {
             'generator' : SyncValidationOutputGenerator,
-            'genCombined': False,
+            'genCombined': True,
+            'regenerate' : True
         },
         'spirv_validation_helper.cpp' : {
             'generator' : SpirvValidationHelperOutputGenerator,
             'genCombined': False,
+            'options' : [grammar],
         },
         'spirv_grammar_helper.h' : {
             'generator' : SpirvGrammarHelperOutputGenerator,
@@ -271,12 +267,12 @@ def RunGenerators(api: str, registry: str, grammar: str, directory: str, styleFi
             'generator' : PnextChainExtractionGenerator,
             'genCombined': True,
         },
-        'state_tracker_helper.h' : {
-            'generator' : StateTrackerHelperOutputGenerator,
+        'device_features.h' : {
+            'generator' : DeviceFeaturesOutputGenerator,
             'genCombined': True,
         },
-        'state_tracker_helper.cpp' : {
-            'generator' : StateTrackerHelperOutputGenerator,
+        'device_features.cpp' : {
+            'generator' : DeviceFeaturesOutputGenerator,
             'genCombined': True,
         },
         'feature_requirements_helper.h' : {
@@ -287,6 +283,10 @@ def RunGenerators(api: str, registry: str, grammar: str, directory: str, styleFi
             'generator' : FeatureRequirementsGenerator,
             'genCombined': True,
         },
+        'test_icd_helper.h' : {
+            'generator' : TestIcdGenerator,
+            'genCombined': False,
+        },
     }
 
     unknownTargets = [x for x in (targetFilter if targetFilter else []) if x not in generators.keys()]
@@ -297,6 +297,11 @@ def RunGenerators(api: str, registry: str, grammar: str, directory: str, styleFi
     # Filter if --target is passed in
     targets = [x for x in generators.keys() if not targetFilter or x in targetFilter]
 
+    cacheVkObjectData = None
+    cachePath = os.path.join(tempfile.gettempdir(), f'vkobject_{os.getpid()}')
+    if caching:
+        EnableCaching()
+
     for index, target in enumerate(targets, start=1):
         print(f'[{index}|{len(targets)}] Generating {target}')
 
@@ -331,13 +336,28 @@ def RunGenerators(api: str, registry: str, grammar: str, directory: str, styleFi
         # Load the XML tree into the registry object
         reg.loadElementTree(tree)
 
-        # Finally, use the output generator to create the requested target
-        reg.apiGen()
+        # The cached data is saved inside the BaseGenerator, so search for it and try
+        # to reuse the parsing for each generator file.
+        if caching and not cacheVkObjectData:
+            if os.path.isfile(cachePath):
+                file = open(cachePath, 'rb')
+                cacheVkObjectData = pickle.load(file)
+                file.close()
+
+        if caching and cacheVkObjectData and ('regenerate' not in generators[target] or not generators[target]['regenerate']):
+            # TODO - We shouldn't have to regenerate any files, need to investigate why we some scripts need it
+            reg.gen.generateFromCache(cacheVkObjectData, reg.genOpts)
+        else:
+            # Finally, use the output generator to create the requested target
+            reg.apiGen()
 
         # Run clang-format on the file
         if has_clang_format:
             common_ci.RunShellCmd(f'clang-format -i --style=file:{styleFile} {os.path.join(directory, target)}')
 
+    if os.path.isfile(cachePath):
+        os.remove(cachePath)
+
 # helper to define paths relative to the repo root
 def repo_relative(path):
     return os.path.abspath(os.path.join(os.path.dirname(__file__), '..', path))
@@ -347,12 +367,21 @@ def main(argv):
     # The shaders requires glslangvalidator, so they are updated manually with generate_spirv when needed
     verify_exclude = [
         '.clang-format',
-        'gpu_as_inspection_comp.h',
-        'gpu_pre_dispatch_comp.h',
-        'gpu_pre_draw_vert.h',
-        'gpu_pre_trace_rays_rgen.h',
-        'inst_functions_comp.h',
-        'gpu_inst_shader_hash.h'
+        'cmd_validation_dispatch_comp.h',
+        'cmd_validation_dispatch_comp.cpp',
+        'cmd_validation_draw_vert.h',
+        'cmd_validation_draw_vert.cpp',
+        'cmd_validation_trace_rays_rgen.h',
+        'cmd_validation_trace_rays_rgen.cpp',
+        'cmd_validation_copy_buffer_to_image_comp.h',
+        'cmd_validation_copy_buffer_to_image_comp.cpp',
+        'instrumentation_buffer_device_address_comp.h',
+        'instrumentation_buffer_device_address_comp.cpp',
+        'instrumentation_bindless_descriptor_comp.h',
+        'instrumentation_bindless_descriptor_comp.cpp',
+        'instrumentation_ray_query_comp.h',
+        'instrumentation_ray_query_comp.cpp',
+        'gpu_av_shader_hash.h'
     ]
 
     parser = argparse.ArgumentParser(description='Generate source code for this repository')
@@ -368,6 +397,7 @@ def main(argv):
     group.add_argument('--target', nargs='+', help='only generate file names passed in')
     group.add_argument('-i', '--incremental', action='store_true', help='only update repo files that change')
     group.add_argument('-v', '--verify', action='store_true', help='verify repo files match generator output')
+    group.add_argument('--no-caching', action='store_true', help='Do not try to cache generator objects')
     args = parser.parse_args(argv)
 
     repo_dir = repo_relative(f'layers/{args.api}/generated')
@@ -411,7 +441,8 @@ def main(argv):
 
     registry = os.path.abspath(os.path.join(args.registry,  'vk.xml'))
     grammar = os.path.abspath(os.path.join(args.grammar, 'spirv.core.grammar.json'))
-    RunGenerators(args.api, registry, grammar, gen_dir, styleFile, args.target)
+    caching = not args.no_caching
+    RunGenerators(args.api, registry, grammar, gen_dir, styleFile, args.target, caching)
 
     # Generate vk_validation_error_messages.h (ignore if targeting a single generator)
     if (not args.target):
