@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2015-2023 The Khronos Group Inc.
- * Copyright (c) 2015-2023 Valve Corporation
- * Copyright (c) 2015-2023 LunarG, Inc.
- * Copyright (c) 2015-2023 Google, Inc.
+ * Copyright (c) 2015-2024 The Khronos Group Inc.
+ * Copyright (c) 2015-2024 Valve Corporation
+ * Copyright (c) 2015-2024 LunarG, Inc.
+ * Copyright (c) 2015-2024 Google, Inc.
  * Modifications Copyright (C) 2020 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,6 +16,8 @@
 #include "../framework/layer_validation_tests.h"
 #include "../framework/pipeline_helper.h"
 
+class NegativeGeometryTessellation : public VkLayerTest {};
+
 TEST_F(NegativeGeometryTessellation, StageMaskGsTsEnabled) {
     TEST_DESCRIPTION(
         "Attempt to use a stageMask w/ geometry shader and tesselation shader bits enabled when those features are disabled on the "
@@ -54,11 +56,11 @@ TEST_F(NegativeGeometryTessellation, StageMaskGsTsEnabled) {
 
     VkCommandBufferBeginInfo cbbi = vku::InitStructHelper();
     vk::BeginCommandBuffer(cmd_buffer, &cbbi);
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdSetEvent-stageMask-04090");
+    m_errorMonitor->SetDesiredError("VUID-vkCmdSetEvent-stageMask-04090");
     vk::CmdSetEvent(cmd_buffer, event, VK_PIPELINE_STAGE_GEOMETRY_SHADER_BIT);
     m_errorMonitor->VerifyFound();
 
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdSetEvent-stageMask-04091");
+    m_errorMonitor->SetDesiredError("VUID-vkCmdSetEvent-stageMask-04091");
     vk::CmdSetEvent(cmd_buffer, event, VK_PIPELINE_STAGE_TESSELLATION_CONTROL_SHADER_BIT);
     m_errorMonitor->VerifyFound();
 
@@ -80,6 +82,7 @@ TEST_F(NegativeGeometryTessellation, GeometryShaderEnabled) {
     }
 
     VkShaderObj vs(this, kVertexMinimalGlsl, VK_SHADER_STAGE_VERTEX_BIT);
+    m_errorMonitor->SetAllowedFailureMsg("VUID-VkShaderModuleCreateInfo-pCode-08740");
     VkShaderObj gs(this, kGeometryMinimalGlsl, VK_SHADER_STAGE_GEOMETRY_BIT);
 
     auto set_info = [&](CreatePipelineHelper &helper) {
@@ -87,8 +90,7 @@ TEST_F(NegativeGeometryTessellation, GeometryShaderEnabled) {
         helper.shader_stages_ = {vs.GetStageCreateInfo(), gs.GetStageCreateInfo(), helper.fs_->GetStageCreateInfo()};
     };
 
-    constexpr std::array vuids = {"VUID-VkPipelineShaderStageCreateInfo-stage-00704", "VUID-VkShaderModuleCreateInfo-pCode-08740"};
-    CreatePipelineHelper::OneshotTest(*this, set_info, kErrorBit, vuids);
+    CreatePipelineHelper::OneshotTest(*this, set_info, kErrorBit, "VUID-VkPipelineShaderStageCreateInfo-stage-00704");
 }
 
 TEST_F(NegativeGeometryTessellation, TessellationShaderEnabled) {
@@ -125,7 +127,9 @@ TEST_F(NegativeGeometryTessellation, TessellationShaderEnabled) {
         }
     )glsl";
 
+    m_errorMonitor->SetAllowedFailureMsg("VUID-VkShaderModuleCreateInfo-pCode-08740");
     VkShaderObj tcs(this, tcsSource, VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT);
+    m_errorMonitor->SetAllowedFailureMsg("VUID-VkShaderModuleCreateInfo-pCode-08740");
     VkShaderObj tes(this, tesSource, VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT);
 
     VkPipelineInputAssemblyStateCreateInfo iasci{VK_STRUCTURE_TYPE_PIPELINE_INPUT_ASSEMBLY_STATE_CREATE_INFO, nullptr, 0,
@@ -140,8 +144,7 @@ TEST_F(NegativeGeometryTessellation, TessellationShaderEnabled) {
         helper.shader_stages_.emplace_back(tcs.GetStageCreateInfo());
         helper.shader_stages_.emplace_back(tes.GetStageCreateInfo());
     };
-    constexpr std::array vuids = {"VUID-VkPipelineShaderStageCreateInfo-stage-00705", "VUID-VkShaderModuleCreateInfo-pCode-08740",
-                                  "VUID-VkShaderModuleCreateInfo-pCode-08740",
+    constexpr std::array vuids = {"VUID-VkPipelineShaderStageCreateInfo-stage-00705",
                                   "VUID-VkPipelineInputAssemblyStateCreateInfo-topology-00430"};
     CreatePipelineHelper::OneshotTest(*this, set_info, kErrorBit, vuids);
 }
@@ -456,42 +459,26 @@ TEST_F(NegativeGeometryTessellation, MaxTessellationControlInputOutputComponents
             "void main(){\n"
             "}\n";
 
-        VkShaderObj tcs(this, tcsSourceStr.c_str(), VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT);
-        VkShaderObj tes(this, kTessellationEvalMinimalGlsl, VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT);
-
-        VkPipelineInputAssemblyStateCreateInfo inputAssemblyInfo = vku::InitStructHelper();
-        inputAssemblyInfo.flags = 0;
-        inputAssemblyInfo.topology = VK_PRIMITIVE_TOPOLOGY_PATCH_LIST;
-        inputAssemblyInfo.primitiveRestartEnable = VK_FALSE;
-
-        VkPipelineTessellationStateCreateInfo tessInfo = vku::InitStructHelper();
-        tessInfo.flags = 0;
-        tessInfo.patchControlPoints = 3;
-
-        m_errorMonitor->SetUnexpectedError("Undefined-Value-ShaderInputNotProduced");
-
-        const auto set_info = [&](CreatePipelineHelper &helper) {
-            helper.shader_stages_ = {helper.vs_->GetStageCreateInfo(), tcs.GetStageCreateInfo(), tes.GetStageCreateInfo(),
-                                     helper.fs_->GetStageCreateInfo()};
-            helper.gp_ci_.pTessellationState = &tessInfo;
-            helper.gp_ci_.pInputAssemblyState = &inputAssemblyInfo;
-        };
         // maxTessellationControlPerVertexInputComponents and maxTessellationControlPerVertexOutputComponents
         switch (overflow) {
             case 0: {
-                CreatePipelineHelper::OneshotTest(*this, set_info, kErrorBit);
+                VkShaderObj tcs(this, tcsSourceStr.c_str(), VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT);
                 break;
             }
             case 1: {
                 // in and out component limit
-                constexpr std::array vuids = {"VUID-RuntimeSpirv-Location-06272", "VUID-RuntimeSpirv-Location-06272"};
-                CreatePipelineHelper::OneshotTest(*this, set_info, kErrorBit, vuids);
+                m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-Location-06272");
+                m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-Location-06272");
+                VkShaderObj tcs(this, tcsSourceStr.c_str(), VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT);
+                m_errorMonitor->VerifyFound();
                 break;
             }
             case 2: {
                 // in and out component limit
-                constexpr std::array vuids = {"VUID-RuntimeSpirv-Location-06272", "VUID-RuntimeSpirv-Location-06272"};
-                CreatePipelineHelper::OneshotTest(*this, set_info, kErrorBit, vuids);
+                m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-Location-06272");
+                m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-Location-06272");
+                VkShaderObj tcs(this, tcsSourceStr.c_str(), VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT);
+                m_errorMonitor->VerifyFound();
                 break;
             }
             default: {
@@ -576,43 +563,26 @@ TEST_F(NegativeGeometryTessellation, MaxTessellationEvaluationInputOutputCompone
             "void main(){\n"
             "}\n";
 
-        VkShaderObj tcs(this, kTessellationControlMinimalGlsl, VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT);
-        VkShaderObj tes(this, tesSourceStr.c_str(), VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT);
-
-        VkPipelineInputAssemblyStateCreateInfo inputAssemblyInfo = vku::InitStructHelper();
-        inputAssemblyInfo.flags = 0;
-        inputAssemblyInfo.topology = VK_PRIMITIVE_TOPOLOGY_PATCH_LIST;
-        inputAssemblyInfo.primitiveRestartEnable = VK_FALSE;
-
-        VkPipelineTessellationStateCreateInfo tessInfo = vku::InitStructHelper();
-        tessInfo.flags = 0;
-        tessInfo.patchControlPoints = 3;
-
-        m_errorMonitor->SetUnexpectedError("Undefined-Value-ShaderInputNotProduced");
-
-        const auto set_info = [&](CreatePipelineHelper &helper) {
-            helper.shader_stages_ = {helper.vs_->GetStageCreateInfo(), tcs.GetStageCreateInfo(), tes.GetStageCreateInfo(),
-                                     helper.fs_->GetStageCreateInfo()};
-            helper.gp_ci_.pTessellationState = &tessInfo;
-            helper.gp_ci_.pInputAssemblyState = &inputAssemblyInfo;
-        };
-
         // maxTessellationEvaluationInputComponents and maxTessellationEvaluationOutputComponents
         switch (overflow) {
             case 0: {
-                CreatePipelineHelper::OneshotTest(*this, set_info, kErrorBit);
+                VkShaderObj tes(this, tesSourceStr.c_str(), VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT);
                 break;
             }
             case 1: {
                 // in and out component limit
-                constexpr std::array vuids = {"VUID-RuntimeSpirv-Location-06272", "VUID-RuntimeSpirv-Location-06272"};
-                CreatePipelineHelper::OneshotTest(*this, set_info, kErrorBit, vuids);
+                m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-Location-06272");
+                m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-Location-06272");
+                VkShaderObj tes(this, tesSourceStr.c_str(), VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT);
+                m_errorMonitor->VerifyFound();
                 break;
             }
             case 2: {
                 // in and out component limit
-                constexpr std::array vuids = {"VUID-RuntimeSpirv-Location-06272", "VUID-RuntimeSpirv-Location-06272"};
-                CreatePipelineHelper::OneshotTest(*this, set_info, kErrorBit, vuids);
+                m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-Location-06272");
+                m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-Location-06272");
+                VkShaderObj tes(this, tesSourceStr.c_str(), VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT);
+                m_errorMonitor->VerifyFound();
                 break;
             }
             default: {
@@ -699,29 +669,26 @@ TEST_F(NegativeGeometryTessellation, MaxGeometryInputOutputComponents) {
                        "void main(){\n"
                        "}\n";
 
-        VkShaderObj gs(this, gsSourceStr.c_str(), VK_SHADER_STAGE_GEOMETRY_BIT);
-
-        m_errorMonitor->SetUnexpectedError("VUID-RuntimeSpirv-OpEntryPoint-08743");
-
-        const auto set_info = [&](CreatePipelineHelper &helper) {
-            helper.shader_stages_ = {helper.vs_->GetStageCreateInfo(), gs.GetStageCreateInfo(), helper.fs_->GetStageCreateInfo()};
-        };
         // maxGeometryInputComponents and maxGeometryOutputComponents
         switch (overflow) {
             case 0: {
-                CreatePipelineHelper::OneshotTest(*this, set_info, kErrorBit);
+                VkShaderObj gs(this, gsSourceStr.c_str(), VK_SHADER_STAGE_GEOMETRY_BIT);
                 break;
             }
             case 1: {
                 // in and out component limit
-                constexpr std::array vuids = {"VUID-RuntimeSpirv-Location-06272", "VUID-RuntimeSpirv-Location-06272"};
-                CreatePipelineHelper::OneshotTest(*this, set_info, kErrorBit, vuids);
+                m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-Location-06272");
+                m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-Location-06272");
+                VkShaderObj gs(this, gsSourceStr.c_str(), VK_SHADER_STAGE_GEOMETRY_BIT);
+                m_errorMonitor->VerifyFound();
                 break;
             }
             case 2: {
                 // in and out component limit
-                constexpr std::array vuids = {"VUID-RuntimeSpirv-Location-06272", "VUID-RuntimeSpirv-Location-06272"};
-                CreatePipelineHelper::OneshotTest(*this, set_info, kErrorBit, vuids);
+                m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-Location-06272");
+                m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-Location-06272");
+                VkShaderObj gs(this, gsSourceStr.c_str(), VK_SHADER_STAGE_GEOMETRY_BIT);
+                m_errorMonitor->VerifyFound();
                 break;
             }
             default: {
@@ -791,7 +758,8 @@ TEST_F(NegativeGeometryTessellation, MaxGeometryInstanceVertexCount) {
     }
 }
 
-TEST_F(NegativeGeometryTessellation, TessellationPatchDecorationMismatch) {
+// Waiting on https://gitlab.khronos.org/vulkan/vulkan/-/issues/3858 to know what is valid or not
+TEST_F(NegativeGeometryTessellation, DISABLED_TessellationPatchDecorationMismatch) {
     TEST_DESCRIPTION(
         "Test that an error is produced for a variable output from the TCS without the patch decoration, but consumed in the TES "
         "with the decoration.");
@@ -884,7 +852,7 @@ TEST_F(NegativeGeometryTessellation, Tessellation) {
     p_iasci = &iasci_bad;
     // Pass a tess control shader without a tess eval shader
     shader_stages = {tcs.GetStageCreateInfo()};
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkGraphicsPipelineCreateInfo-pStages-09022");
+    m_errorMonitor->SetDesiredError("VUID-VkGraphicsPipelineCreateInfo-pStages-09022");
     CreatePipelineHelper::OneshotTest(*this, set_info, kErrorBit, "VUID-VkGraphicsPipelineCreateInfo-pStages-00729");
 
     // Pass a tess eval shader without a tess control shader
@@ -948,7 +916,7 @@ TEST_F(NegativeGeometryTessellation, PatchControlPoints)
     // Attempt to Create Gfx Pipeline w/o a VS
     VkResult err;
 
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "Invalid Pipeline CreateInfo State: VK_PRIMITIVE_TOPOLOGY_PATCH primitive ");
+    m_errorMonitor->SetDesiredError("Invalid Pipeline CreateInfo State: VK_PRIMITIVE_TOPOLOGY_PATCH primitive ");
 
     RETURN_IF_SKIP(Init());
     InitRenderTarget();
@@ -976,7 +944,7 @@ TEST_F(NegativeGeometryTessellation, PatchControlPoints)
     vkt::DescriptorSetLayout ds_layout(*m_device, ds_layout_ci);
 
     VkDescriptorSet descriptorSet;
-    err = vk::AllocateDescriptorSets(m_device->device(), ds_pool.handle(),
+    err = vk::AllocateDescriptorSets(device(), ds_pool.handle(),
 VK_DESCRIPTOR_SET_USAGE_NON_FREE, 1, &ds_layout.handle(), &descriptorSet);
     ASSERT_EQ(VK_SUCCESS, err);
 
@@ -1034,15 +1002,15 @@ VK_DESCRIPTOR_SET_USAGE_NON_FREE, 1, &ds_layout.handle(), &descriptorSet);
     VkPipeline pipeline;
     VkPipelineCache pipelineCache;
 
-    err = vk::CreatePipelineCache(m_device->device(), &pc_ci, NULL,
+    err = vk::CreatePipelineCache(device(), &pc_ci, NULL,
 &pipelineCache);
     ASSERT_EQ(VK_SUCCESS, err);
-    err = vk::CreateGraphicsPipelines(m_device->device(), pipelineCache, 1,
+    err = vk::CreateGraphicsPipelines(device(), pipelineCache, 1,
 &gp_ci, NULL, &pipeline);
 
     m_errorMonitor->VerifyFound();
 
-    vk::DestroyPipelineCache(m_device->device(), pipelineCache, NULL);
+    vk::DestroyPipelineCache(device(), pipelineCache, NULL);
 }
 */
 
@@ -1184,12 +1152,11 @@ TEST_F(NegativeGeometryTessellation, PipelineTessellationMissingPointSize) {
     tess_ci.patchControlPoints = 4u;
 
     CreatePipelineHelper pipe(*this);
-    pipe.InitState();
     pipe.shader_stages_ = {pipe.vs_->GetStageCreateInfo(), tcs.GetStageCreateInfo(), tes.GetStageCreateInfo(),
                            pipe.fs_->GetStageCreateInfo()};
     pipe.ia_ci_.topology = VK_PRIMITIVE_TOPOLOGY_PATCH_LIST;
     pipe.tess_ci_ = tess_ci;
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkGraphicsPipelineCreateInfo-TessellationEvaluation-07723");
+    m_errorMonitor->SetDesiredError("VUID-VkGraphicsPipelineCreateInfo-TessellationEvaluation-07723");
     pipe.CreateGraphicsPipeline();
     m_errorMonitor->VerifyFound();
 }
@@ -1231,20 +1198,20 @@ TEST_F(NegativeGeometryTessellation, PipelineTessellationPointSize) {
         }
     )glsl";
 
+    m_errorMonitor->SetAllowedFailureMsg("VUID-VkShaderModuleCreateInfo-pCode-08740");
     VkShaderObj tcs(this, kTessellationControlMinimalGlsl, VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT);
+    m_errorMonitor->SetAllowedFailureMsg("VUID-VkShaderModuleCreateInfo-pCode-08740");
     VkShaderObj tes(this, tess_src, VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT);
 
     VkPipelineTessellationStateCreateInfo tess_ci = vku::InitStructHelper();
     tess_ci.patchControlPoints = 4u;
 
     CreatePipelineHelper pipe(*this);
-    pipe.InitState();
     pipe.shader_stages_ = {pipe.vs_->GetStageCreateInfo(), tcs.GetStageCreateInfo(), tes.GetStageCreateInfo(),
                            pipe.fs_->GetStageCreateInfo()};
     pipe.ia_ci_.topology = VK_PRIMITIVE_TOPOLOGY_PATCH_LIST;
     pipe.tess_ci_ = tess_ci;
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkShaderModuleCreateInfo-pCode-08740");
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkGraphicsPipelineCreateInfo-TessellationEvaluation-07724");
+    m_errorMonitor->SetDesiredError("VUID-VkGraphicsPipelineCreateInfo-TessellationEvaluation-07724");
     pipe.CreateGraphicsPipeline();
     m_errorMonitor->VerifyFound();
 }
@@ -1340,14 +1307,13 @@ TEST_F(NegativeGeometryTessellation, GeometryStreamsCapability) {
                OpFunctionEnd
     )glsl";
 
+    m_errorMonitor->SetAllowedFailureMsg("VUID-VkShaderModuleCreateInfo-pCode-08740");
     VkShaderObj gs(this, geom_src, VK_SHADER_STAGE_GEOMETRY_BIT, SPV_ENV_VULKAN_1_0, SPV_SOURCE_ASM);
 
     CreatePipelineHelper pipe(*this);
-    pipe.InitState();
     pipe.shader_stages_ = {pipe.vs_->GetStageCreateInfo(), gs.GetStageCreateInfo(), pipe.fs_->GetStageCreateInfo()};
     pipe.ia_ci_.topology = VK_PRIMITIVE_TOPOLOGY_POINT_LIST;
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkShaderModuleCreateInfo-pCode-08740");
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkGraphicsPipelineCreateInfo-geometryStreams-02321");
+    m_errorMonitor->SetDesiredError("VUID-VkGraphicsPipelineCreateInfo-geometryStreams-02321");
     pipe.CreateGraphicsPipeline();
     m_errorMonitor->VerifyFound();
 }
@@ -1540,13 +1506,46 @@ TEST_F(NegativeGeometryTessellation, MismatchedTessellationExecutionModes) {
         tess_ci.patchControlPoints = 4u;
 
         CreatePipelineHelper pipe(*this);
-        pipe.InitState();
         pipe.ia_ci_.topology = VK_PRIMITIVE_TOPOLOGY_PATCH_LIST;
         pipe.tess_ci_ = tess_ci;
         pipe.shader_stages_ = {pipe.vs_->GetStageCreateInfo(), tesc.GetStageCreateInfo(), tese.GetStageCreateInfo(),
                                pipe.fs_->GetStageCreateInfo()};
-        m_errorMonitor->SetDesiredFailureMsg(kErrorBit, vuids[i]);
+        m_errorMonitor->SetDesiredError(vuids[i].c_str());
         pipe.CreateGraphicsPipeline();
         m_errorMonitor->VerifyFound();
     }
 }
+
+TEST_F(NegativeGeometryTessellation, WritingToLayerWithSingleFramebufferLayer) {
+    TEST_DESCRIPTION("https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/3019");
+    RETURN_IF_SKIP(Init());
+    InitRenderTarget();  // Creates a framebuffer with a single layer
+    if (m_device->phy().limits_.maxGeometryOutputVertices == 0) {
+        GTEST_SKIP() << "Device doesn't support required maxGeometryOutputVertices";
+    }
+    const std::string_view gsSource = R"glsl(
+        #version 450
+        layout (triangles) in;
+        layout (triangle_strip) out;
+        layout (max_vertices = 1) out;
+        void main() {
+            gl_Position = vec4(1.0, 0.5, 0.5, 0.0);
+            EmitVertex();
+            gl_Layer = 4;
+        }
+    )glsl";
+    const VkShaderObj gs(this, gsSource.data(), VK_SHADER_STAGE_GEOMETRY_BIT);
+
+    CreatePipelineHelper pipe(*this);
+    pipe.shader_stages_ = {pipe.vs_->GetStageCreateInfo(), gs.GetStageCreateInfo(), pipe.fs_->GetStageCreateInfo()};
+    pipe.CreateGraphicsPipeline();
+
+    m_commandBuffer->begin();
+    m_commandBuffer->BeginRenderPass(m_renderPassBeginInfo);
+    vk::CmdBindPipeline(m_commandBuffer->handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipe.Handle());
+    m_errorMonitor->SetDesiredWarning("Undefined-Layer-Written");
+    vk::CmdDraw(m_commandBuffer->handle(), 3, 1, 0, 0);
+    m_errorMonitor->VerifyFound();
+    m_commandBuffer->EndRenderPass();
+    m_commandBuffer->end();
+}
\ No newline at end of file
