@@ -1,5 +1,5 @@
-/* Copyright (c) 2021-2023 The Khronos Group Inc.
- * Copyright (c) 2023 LunarG, Inc.
+/* Copyright (c) 2021-2024 The Khronos Group Inc.
+ * Copyright (c) 2023-2024 LunarG, Inc.
  * Copyright (c) 2021 NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -26,7 +26,7 @@ class ViewportInheritanceTestData {
     VkRenderPass m_renderPass{};
 
     // Framebuffer data.
-    VkImageObj m_colorImageObj;
+    vkt::Image m_colorImageObj;
     VkImageView m_colorImageView{};
     VkFramebuffer m_framebuffer{};
 
@@ -102,9 +102,9 @@ class ViewportInheritanceTestData {
         if (result != VK_SUCCESS) m_failureReason = "Could not create render pass.";
     }
 
-    void CreateColorImageObj() noexcept {
+    void CreateColorImageObj(vkt::Device& device_obj) noexcept {
         assert(m_colorFormat != VK_FORMAT_UNDEFINED);
-        m_colorImageObj.Init(128, 128, 1, m_colorFormat, VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT);
+        m_colorImageObj.Init(device_obj, 128, 128, 1, m_colorFormat, VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT);
         if (!m_colorImageObj.initialized()) {
             m_failureReason = "Image not initialized";
         }
@@ -246,15 +246,14 @@ class ViewportInheritanceTestData {
         return true;
     }
 
-    ViewportInheritanceTestData(vkt::Device* p_device_obj, VkPhysicalDevice physical_device) noexcept
-        : m_colorImageObj(p_device_obj) {
+    ViewportInheritanceTestData(vkt::Device* p_device_obj, VkPhysicalDevice physical_device) noexcept {
         m_device = p_device_obj->handle();
 
         PickColorFormat(physical_device);
         if (m_failureReason) return;
         CreateRenderPass();
         if (m_failureReason) return;
-        CreateColorImageObj();
+        CreateColorImageObj(*p_device_obj);
         if (m_failureReason) return;
         CreateColorView();
         if (m_failureReason) return;
@@ -400,28 +399,28 @@ class ViewportInheritanceTestData {
     }
 };
 
+class NegativeViewportInheritance : public VkLayerTest {};
+
 TEST_F(NegativeViewportInheritance, BasicUsage) {
 #if defined(VVL_ENABLE_TSAN)
     GTEST_SKIP() << "https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/5965";
 #endif
     TEST_DESCRIPTION("Simple correct and incorrect usage of VK_NV_inherited_viewport_scissor");
-    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
-    RETURN_IF_SKIP(InitFramework());
-    bool has_features = false;
-    const char* missing_feature_string = nullptr;
-    auto self = this;
-    RETURN_IF_SKIP(has_features = ViewportInheritanceTestData::InitState(
-                       this, [self](const char* extension) { self->m_device_extension_names.push_back(extension); },
-                       &missing_feature_string, true, false));
-    if (!has_features) {
-        GTEST_SKIP() << missing_feature_string;
-    }
+    AddRequiredExtensions(VK_EXT_EXTENDED_DYNAMIC_STATE_EXTENSION_NAME);
+    AddRequiredExtensions(VK_NV_INHERITED_VIEWPORT_SCISSOR_EXTENSION_NAME);
+    AddRequiredExtensions(VK_EXT_NESTED_COMMAND_BUFFER_EXTENSION_NAME);
+    AddRequiredFeature(vkt::Feature::multiViewport);
+    AddRequiredFeature(vkt::Feature::extendedDynamicState);
+    AddRequiredFeature(vkt::Feature::inheritedViewportScissor2D);
+    AddRequiredFeature(vkt::Feature::nestedCommandBuffer);
+    AddRequiredFeature(vkt::Feature::nestedCommandBufferSimultaneousUse);
+    RETURN_IF_SKIP(Init());
 
     ViewportInheritanceTestData test_data(m_device, gpu());
     if (test_data.FailureReason()) {
         GTEST_SKIP() << "Test internal failure: " << test_data.FailureReason();
     }
-    VkCommandPool pool = m_commandPool->handle();
+    VkCommandPool pool = m_command_pool.handle();
 
     VkCommandBuffer subpass_cmd = test_data.MakeBeginSubpassCommandBuffer(pool, 1, test_data.kViewportDepthOnlyArray);
     test_data.BindGraphicsPipeline(subpass_cmd, true, 1);
@@ -438,7 +437,7 @@ TEST_F(NegativeViewportInheritance, BasicUsage) {
     vk::EndCommandBuffer(primary_cmd);
 
     // Viewport with incorrect depth range.
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");
+    m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");
     test_data.BeginPrimaryCommandBuffer(primary_cmd);
     vk::CmdSetViewport(primary_cmd, 0, 1, test_data.kViewportAlternateDepthArray);
     vk::CmdSetScissor(primary_cmd, 0, 1, test_data.kScissorArray);
@@ -449,7 +448,7 @@ TEST_F(NegativeViewportInheritance, BasicUsage) {
     m_errorMonitor->VerifyFound();
 
     // Viewport not provided.
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");
+    m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");
     test_data.BeginPrimaryCommandBuffer(primary_cmd);
     vk::CmdSetScissor(primary_cmd, 0, 1, test_data.kScissorArray);
     test_data.BeginRenderPass(primary_cmd);
@@ -459,7 +458,7 @@ TEST_F(NegativeViewportInheritance, BasicUsage) {
     m_errorMonitor->VerifyFound();
 
     // Scissor not provided.
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");
+    m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");
     test_data.BeginPrimaryCommandBuffer(primary_cmd);
     vk::CmdSetViewport(primary_cmd, 0, 1, test_data.kViewportArray);
     test_data.BeginRenderPass(primary_cmd);
@@ -472,8 +471,8 @@ TEST_F(NegativeViewportInheritance, BasicUsage) {
     // Don't swap the loop order or you'll mess up subpass_cmd for upcoming tests.
     for (int should_fail = 1; should_fail >= 0; --should_fail) {
         if (should_fail) {
-            m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07832");  // viewport
-            m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07831");  // scissor
+            m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07832");  // viewport
+            m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07831");  // scissor
             test_data.BeginSubpassCommandBuffer(subpass_cmd, 0, nullptr);
         } else {
             test_data.BeginSubpassCommandBuffer(subpass_cmd, 1, test_data.kViewportDepthOnlyArray);
@@ -504,8 +503,8 @@ TEST_F(NegativeViewportInheritance, BasicUsage) {
     // trashed afterwards).
     for (int should_fail = 0; should_fail < 2; ++should_fail) {
         if (should_fail) {
-            m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");  // viewport
-            m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");  // scissor
+            m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");  // viewport
+            m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");  // scissor
         }
         std::array<VkCommandBuffer, 2> secondaries = {should_fail ? static_state_cmd : subpass_cmd,
                                                       should_fail ? subpass_cmd : static_state_cmd};
@@ -526,8 +525,8 @@ TEST_F(NegativeViewportInheritance, BasicUsage) {
     // viewport/scissor pipeline.
     for (int should_fail = 0; should_fail < 2; ++should_fail) {
         if (should_fail) {
-            m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");  // viewport
-            m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");  // scissor
+            m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");  // viewport
+            m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");  // scissor
         }
 
         test_data.BeginPrimaryCommandBuffer(primary_cmd);
@@ -550,7 +549,7 @@ TEST_F(NegativeViewportInheritance, BasicUsage) {
 
     for (int should_fail = 0; should_fail < 2; ++should_fail) {
         if (should_fail) {
-            m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");  // viewport
+            m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");  // viewport
         } else {
         }
 
@@ -568,16 +567,16 @@ TEST_F(NegativeViewportInheritance, BasicUsage) {
     // Check that the validation layers are not okay with binding static viewport/scissor pipelines when inheritance enabled, or
     // setting viewport/scissor explicitly, but are okay if inheritance is not enabled (no regression).
     for (int should_fail = 0; should_fail < 2; ++should_fail) {
-        if (should_fail) m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdBindPipeline-commandBuffer-04808");
+        if (should_fail) m_errorMonitor->SetDesiredError("VUID-vkCmdBindPipeline-commandBuffer-04808");
         test_data.BeginSubpassCommandBuffer(no_draw_cmd, should_fail, test_data.kViewportDepthOnlyArray);
         test_data.BindGraphicsPipeline(no_draw_cmd, false, 1);
         if (should_fail) m_errorMonitor->VerifyFound();
 
         // Check that the validation layers flag setting viewport/scissor with inheritance.
-        if (should_fail) m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdSetViewport-commandBuffer-04821");
+        if (should_fail) m_errorMonitor->SetDesiredError("VUID-vkCmdSetViewport-commandBuffer-04821");
         vk::CmdSetViewport(no_draw_cmd, 0, 1, test_data.kViewportArray);
         if (should_fail) m_errorMonitor->VerifyFound();
-        if (should_fail) m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdSetScissor-viewportScissor2D-04789");
+        if (should_fail) m_errorMonitor->SetDesiredError("VUID-vkCmdSetScissor-viewportScissor2D-04789");
         vk::CmdSetScissor(no_draw_cmd, 0, 1, test_data.kScissorArray);
         if (should_fail) m_errorMonitor->VerifyFound();
 
@@ -587,8 +586,7 @@ TEST_F(NegativeViewportInheritance, BasicUsage) {
     // Check for at least 1 viewport depth given when enabling inheritance.
     for (int should_fail = 0; should_fail < 2; ++should_fail) {
         if (should_fail) {
-            m_errorMonitor->SetDesiredFailureMsg(kErrorBit,
-                                                 "VUID-VkCommandBufferInheritanceViewportScissorInfoNV-viewportScissor2D-04784");
+            m_errorMonitor->SetDesiredError("VUID-VkCommandBufferInheritanceViewportScissorInfoNV-viewportScissor2D-04784");
         } else {
         }
         VkCommandBufferInheritanceViewportScissorInfoNV viewport_scissor = {
@@ -603,7 +601,7 @@ TEST_F(NegativeViewportInheritance, BasicUsage) {
     }
 
     // Check for VK_COMMAND_BUFFER_USAGE_RENDER_PASS_CONTINUE_BIT
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkCommandBufferInheritanceViewportScissorInfoNV-viewportScissor2D-04786");
+    m_errorMonitor->SetDesiredError("VUID-VkCommandBufferInheritanceViewportScissorInfoNV-viewportScissor2D-04786");
     VkCommandBufferInheritanceViewportScissorInfoNV viewport_scissor = {
         VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_VIEWPORT_SCISSOR_INFO_NV, nullptr, VK_TRUE, 1, test_data.kViewportArray};
     test_data.MakeBeginSecondaryCommandBuffer(pool, 0, &viewport_scissor);
@@ -622,7 +620,7 @@ TEST_F(NegativeViewportInheritance, BasicUsage) {
         test_data.BeginPrimaryCommandBuffer(primary_cmd);
         test_data.BeginRenderPass(primary_cmd);
         if (should_fail) {
-            m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");  // viewport
+            m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");  // viewport
             vk::CmdExecuteCommands(primary_cmd, 1, &set_viewport_cmd);
             VkCommandBuffer secondaries[2] = {set_scissor_cmd, subpass_cmd};
             vk::CmdExecuteCommands(primary_cmd, 2, secondaries);
@@ -639,7 +637,7 @@ TEST_F(NegativeViewportInheritance, BasicUsage) {
 
 TEST_F(NegativeViewportInheritance, MissingFeature) {
     TEST_DESCRIPTION("Error using VK_NV_inherited_viewport_scissor without enabling feature.");
-    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
+    AddRequiredExtensions(VK_EXT_EXTENDED_DYNAMIC_STATE_EXTENSION_NAME);
     RETURN_IF_SKIP(InitFramework());
     bool has_features = false;
     const char* missing_feature_string = nullptr;
@@ -655,11 +653,11 @@ TEST_F(NegativeViewportInheritance, MissingFeature) {
     if (test_data.FailureReason()) {
         GTEST_SKIP() << "Test internal failure: " << test_data.FailureReason();
     }
-    VkCommandPool pool = m_commandPool->handle();
+    VkCommandPool pool = m_command_pool.handle();
 
     test_data.MakeBeginSubpassCommandBuffer(pool, 0, nullptr);
 
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkCommandBufferInheritanceViewportScissorInfoNV-viewportScissor2D-04782");
+    m_errorMonitor->SetDesiredError("VUID-VkCommandBufferInheritanceViewportScissorInfoNV-viewportScissor2D-04782");
     test_data.MakeBeginSubpassCommandBuffer(pool, 1, test_data.kViewportArray);
     m_errorMonitor->VerifyFound();
 }
@@ -669,27 +667,25 @@ TEST_F(NegativeViewportInheritance, MultiViewport) {
     GTEST_SKIP() << "https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/5965";
 #endif
     TEST_DESCRIPTION("VK_NV_inherited_viewport_scissor tests with multiple viewports/scissors");
-    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
-    RETURN_IF_SKIP(InitFramework());
-    bool has_features = false;
-    const char* missing_feature_string = nullptr;
-    auto self = this;
-    RETURN_IF_SKIP(has_features = ViewportInheritanceTestData::InitState(
-                       this, [self](const char* extension) { self->m_device_extension_names.push_back(extension); },
-                       &missing_feature_string, true, true));
-    if (!has_features) {
-        GTEST_SKIP() << missing_feature_string;
-    }
+    AddRequiredExtensions(VK_EXT_EXTENDED_DYNAMIC_STATE_EXTENSION_NAME);
+    AddRequiredExtensions(VK_NV_INHERITED_VIEWPORT_SCISSOR_EXTENSION_NAME);
+    AddRequiredExtensions(VK_EXT_NESTED_COMMAND_BUFFER_EXTENSION_NAME);
+    AddRequiredFeature(vkt::Feature::multiViewport);
+    AddRequiredFeature(vkt::Feature::extendedDynamicState);
+    AddRequiredFeature(vkt::Feature::inheritedViewportScissor2D);
+    AddRequiredFeature(vkt::Feature::nestedCommandBuffer);
+    AddRequiredFeature(vkt::Feature::nestedCommandBufferSimultaneousUse);
+    RETURN_IF_SKIP(Init());
 
     ViewportInheritanceTestData test_data(m_device, gpu());
     if (test_data.FailureReason()) {
         GTEST_SKIP() << "Test internal failure: " << test_data.FailureReason();
     }
-    VkCommandPool pool = m_commandPool->handle();
+    VkCommandPool pool = m_command_pool.handle();
 
     // Test using viewport/scissor with count state without providing it to be inherited.
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");  // viewport
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");  // scissor
+    m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");  // viewport
+    m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");  // scissor
     VkCommandBuffer draw_cmd = test_data.MakeBeginSubpassCommandBuffer(pool, 1, test_data.kViewportArray);
     test_data.BindGraphicsPipeline(draw_cmd, true, 0 /* dynamic viewport and scissor count */);
     vk::CmdDraw(draw_cmd, 3, 1, 0, 0);
@@ -707,7 +703,7 @@ TEST_F(NegativeViewportInheritance, MultiViewport) {
     // Test drawing with pipeline that uses more viewports than have been inherited.
     for (int i = 0; i < 4; ++i) {
         auto should_fail = i & 1;
-        if (should_fail) m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");  // viewport
+        if (should_fail) m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");  // viewport
 
         test_data.BeginSubpassCommandBuffer(draw_cmd, should_fail ? 1 : 2, test_data.kViewportDepthOnlyArray);
         test_data.BindGraphicsPipeline(draw_cmd, true, 2); // Uses 2 viewports
@@ -751,11 +747,11 @@ TEST_F(NegativeViewportInheritance, MultiViewport) {
         auto should_fail = i & 1;
         auto use_with_count = i & 4;
         if (should_fail) {
-            m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");  // viewport 0 (or with count)
-            m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");  // scissor 0 (or with count)
+            m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");  // viewport 0 (or with count)
+            m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");  // scissor 0 (or with count)
             if (!use_with_count) {
-                m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");  // viewport 1
-                m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");  // scissor 1
+                m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");  // viewport 1
+                m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");  // scissor 1
             }
         } else {
         }
@@ -810,7 +806,7 @@ TEST_F(NegativeViewportInheritance, MultiViewport) {
     for (int i = 0; i < 3; ++i) {
         auto should_fail = i & 1;
         if (should_fail) {
-            m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");  // viewport 1
+            m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");  // viewport 1
         } else {
         }
 
@@ -843,7 +839,7 @@ TEST_F(NegativeViewportInheritance, MultiViewport) {
         auto should_fail = i & 2;
         auto inherited_incorrect_depth = i & 4;
 
-        if (should_fail) m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdDraw-None-07850");
+        if (should_fail) m_errorMonitor->SetDesiredError("VUID-vkCmdDraw-None-07850");
 
         VkViewport viewports[3];
         viewports[0] = test_data.kViewportArray[0];
@@ -899,7 +895,7 @@ TEST_F(NegativeViewportInheritance, MultiViewport) {
 
 TEST_F(NegativeViewportInheritance, ScissorMissingFeature) {
     TEST_DESCRIPTION("Error using VK_NV_inherited_viewport_scissor without enabling multiViewport feature.");
-    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
+    AddRequiredExtensions(VK_EXT_EXTENDED_DYNAMIC_STATE_EXTENSION_NAME);
     RETURN_IF_SKIP(InitFramework());
     bool has_features = false;
     const char* missing_feature_string = nullptr;
@@ -915,17 +911,16 @@ TEST_F(NegativeViewportInheritance, ScissorMissingFeature) {
     if (test_data.FailureReason()) {
         GTEST_SKIP() << "Test internal failure: " << test_data.FailureReason();
     }
-    VkCommandPool pool = m_commandPool->handle();
+    VkCommandPool pool = m_command_pool.handle();
 
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkCommandBufferInheritanceViewportScissorInfoNV-viewportScissor2D-04783");
+    m_errorMonitor->SetDesiredError("VUID-VkCommandBufferInheritanceViewportScissorInfoNV-viewportScissor2D-04783");
     test_data.MakeBeginSubpassCommandBuffer(pool, 2, test_data.kViewportArray);
     m_errorMonitor->VerifyFound();
 }
 
 TEST_F(NegativeViewportInheritance, PipelineMissingDynamicStateDiscardRectangle) {
     TEST_DESCRIPTION("Bind pipeline with missing dynamic state discard rectangle.");
-
-    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
+    AddRequiredExtensions(VK_EXT_EXTENDED_DYNAMIC_STATE_EXTENSION_NAME);
     AddRequiredExtensions(VK_EXT_DISCARD_RECTANGLES_EXTENSION_NAME);
     RETURN_IF_SKIP(InitFramework());
 
@@ -948,7 +943,7 @@ TEST_F(NegativeViewportInheritance, PipelineMissingDynamicStateDiscardRectangle)
     InitRenderTarget();
 
     vkt::CommandPool pool(*m_device, m_device->graphics_queue_node_index_, VK_COMMAND_POOL_CREATE_RESET_COMMAND_BUFFER_BIT);
-    vkt::CommandBuffer secondary(m_device, &pool, VK_COMMAND_BUFFER_LEVEL_SECONDARY);
+    vkt::CommandBuffer secondary(*m_device, pool, VK_COMMAND_BUFFER_LEVEL_SECONDARY);
 
     ViewportInheritanceTestData test_data(m_device, gpu());
     if (test_data.FailureReason()) {
@@ -979,16 +974,14 @@ TEST_F(NegativeViewportInheritance, PipelineMissingDynamicStateDiscardRectangle)
     dyn_state_ci.dynamicStateCount = 2;
     dyn_state_ci.pDynamicStates = dyn_states;
 
-    CreatePipelineHelper pipe(*this);
-    pipe.gp_ci_.pNext = &discard_rectangle_state;
+    CreatePipelineHelper pipe(*this, &discard_rectangle_state);
     pipe.gp_ci_.pDynamicState = &dyn_state_ci;
-    pipe.InitState();
     pipe.CreateGraphicsPipeline();
 
     vk::BeginCommandBuffer(secondary.handle(), &cbbi);
 
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdBindPipeline-commandBuffer-04809");
-    vk::CmdBindPipeline(secondary.handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipe.pipeline_);
+    m_errorMonitor->SetDesiredError("VUID-vkCmdBindPipeline-commandBuffer-04809");
+    vk::CmdBindPipeline(secondary.handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipe.Handle());
     m_errorMonitor->VerifyFound();
 
     if (DeviceExtensionSupported(VK_EXT_DISCARD_RECTANGLES_EXTENSION_NAME, 2)) {
@@ -996,10 +989,9 @@ TEST_F(NegativeViewportInheritance, PipelineMissingDynamicStateDiscardRectangle)
 
         CreatePipelineHelper pipe2(*this);
         pipe2.gp_ci_.pDynamicState = &dyn_state_ci;
-        pipe2.InitState();
         pipe2.CreateGraphicsPipeline();
-        m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkCmdBindPipeline-commandBuffer-04809");
-        vk::CmdBindPipeline(secondary.handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipe2.pipeline_);
+        m_errorMonitor->SetDesiredError("VUID-vkCmdBindPipeline-commandBuffer-04809");
+        vk::CmdBindPipeline(secondary.handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipe2.Handle());
         m_errorMonitor->VerifyFound();
     }
 }
